@@ -26,6 +26,11 @@
 #include <vector>
 #include <string>
 #include <iostream>
+#include <valarray>
+
+#include <set>
+#include <boost/tuple/tuple.hpp>
+
 
 namespace GAGCore
 {
@@ -62,7 +67,9 @@ namespace GAGCore
 		bool operator==(const Color &o) const { return pack() == o.pack(); }
 		
 		//! return a new color with a different alpha value
-		Color applyAlpha(Uint8 a) { Color c = *this; c.a = a; return c; }
+		Color applyAlpha(Uint8 _a) const { Color c = *this; c.a = _a; return c; }
+		//! return a new color resulting of the multiplication by alpha
+		Color applyMultiplyAlpha(Uint8 _a) const ;
 		
 		static Color black; //!< black color (0,0,0)
 		static Color white; //!< black color (255,255,255)
@@ -196,7 +203,7 @@ namespace GAGCore
 		virtual void getClipRect(int *x, int *y, int *w, int *h);
 		virtual void setClipRect(int x, int y, int w, int h);
 		virtual void setClipRect(void);
-		virtual void nextFrame(void) { }
+		virtual void nextFrame(void) { flushTextPictures(); }
 		virtual bool loadImage(const char *name);
 		virtual bool loadImage(const std::string &name);
 		virtual void shiftHSV(float hue, float sat, float lum);
@@ -248,6 +255,10 @@ namespace GAGCore
 		void drawString(int x, int y, Font *font, const std::string &msg, int w = 0, Uint8 alpha = Color::ALPHA_OPAQUE);
 		void drawString(float x, float y, Font *font, const std::string &msg, float w = 0, Uint8 alpha = Color::ALPHA_OPAQUE);
 		
+		//! Draw an alpha map of size mapW, mapH using a specific color at coordinantes x, y using cells of size cellW, cellH
+		virtual void drawAlphaMap(const std::valarray<float> &map, int mapW, int mapH, int x, int y, int cellW, int cellH, const Color &color);
+		virtual void drawAlphaMap(const std::valarray<unsigned char> &map, int mapW, int mapH, int x, int y, int cellW, int cellH, const Color &color);
+		
 		// old API, deprecated, do not use. It is only there for compatibility with existing code
 		virtual void drawPixel(int x, int y, Uint8 r, Uint8 g, Uint8 b, Uint8 a = Color::ALPHA_OPAQUE);
 		virtual void drawRect(int x, int y, int w, int h, Uint8 r, Uint8 g, Uint8 b, Uint8 a = Color::ALPHA_OPAQUE);
@@ -257,6 +268,31 @@ namespace GAGCore
 		virtual void drawLine(int x1, int y1, int x2, int y2, Uint8 r, Uint8 g, Uint8 b, Uint8 a = Color::ALPHA_OPAQUE);
 		virtual void drawCircle(int x, int y, int radius, Uint8 r, Uint8 g, Uint8 b, Uint8 a = Color::ALPHA_OPAQUE);
 		virtual void drawString(int x, int y, Font *font, int i);
+		
+		// This is for translation textshot code, it works by trapping calls to the getString function in the translation StringTables,
+		// then later in drawString, if we are drawing one of the found strings returned by StringTable, it will add it to the list of
+		// rectangles that represent found texts. Just before the next frame begins to draw, all of the rectangle pictures are flushed
+		// into bmp's. This is done because we want the translation pictures to be done when *all* of the screen is already drawn (when
+		// we flushed into a bmp directly from drawString, sometimes the screen was only partially drawn, and the resulting pictures
+		// would be hard to interpret)
+		class SRectangle
+		{
+		public:
+			SRectangle(int x, int y, int w, int h) : x(x), y(y), w(w), h(h) {}
+			int x, y, w, h;
+		};
+		// This holds texts requested by the toolkit, but not yet drawn to the screen
+		static std::map<std::string, std::string> texts;
+		// This holds texts already drawn to the screen and detected, so that pictures aren't taken twice, which would lag the game badly
+		static std::set<std::string> wroteTexts;
+		// This holds detected texts that will be printed on the next flush
+		static std::vector<boost::tuple<SRectangle, std::string, GAGCore::DrawableSurface*> > drawSquares;
+		// This holds the directory the pictures will be stored in. The system is disabled if this string is empty.
+		static std::string translationPicturesDirectory;
+		// This flushes all of the detected texts, making bmp pictures
+		static void flushTextPictures();
+		// This prints out information on all of the texts that where requested but not detected
+		static void printFinishingText();
 	};
 	
 	//! A GraphicContext is a DrawableSurface that represent the main screen of the application.
@@ -335,6 +371,9 @@ namespace GAGCore
 		virtual void drawSurface(int x, int y, int w, int h, DrawableSurface *surface, int sx, int sy, int sw, int sh,  Uint8 alpha = Color::ALPHA_OPAQUE);
 		virtual void drawSurface(float x, float y, float w, float h, DrawableSurface *surface, int sx, int sy, int sw, int sh, Uint8 alpha = Color::ALPHA_OPAQUE);
 		
+		virtual void drawAlphaMap(const std::valarray<float> &map, int mapW, int mapH, int x, int y, int cellW, int cellH, const Color &color);
+		virtual void drawAlphaMap(const std::valarray<unsigned char> &map, int mapW, int mapH, int x, int y, int cellW, int cellH, const Color &color);
+		
 		// compat
 		virtual void drawPixel(int x, int y, Uint8 r, Uint8 g, Uint8 b, Uint8 a = Color::ALPHA_OPAQUE);
 		virtual void drawRect(int x, int y, int w, int h, Uint8 r, Uint8 g, Uint8 b, Uint8 a = Color::ALPHA_OPAQUE);
