@@ -30,6 +30,7 @@
 #include <math.h>
 #include <string.h>
 #include <valarray>
+#include <cstdlib>
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -85,6 +86,16 @@ namespace GAGCore
 		b = static_cast<Uint8>(255.0f*fb);
 	}
 	
+	Color Color::applyMultiplyAlpha(Uint8 _a) const 
+	{
+		Color c;
+		c.r = (Uint8)(((unsigned)r * (unsigned)_a) / 256);
+		c.g = (Uint8)(((unsigned)g * (unsigned)_a) / 256);
+		c.b = (Uint8)(((unsigned)b * (unsigned)_a) / 256);
+		c.a = (Uint8)(((unsigned)a * (unsigned)_a) / 256);
+		return c;
+	}
+	
 	// Predefined colors
 	Color Color::black = Color(0, 0, 0);
 	Color Color::white = Color(255,255,255);
@@ -99,14 +110,16 @@ namespace GAGCore
 		int _doScissor;
 		GLint _texture;
 		GLenum _sfactor, _dfactor;
-		bool isTextureRectangle;
+		bool isTextureSRectangle;
 		bool useATIWorkaround;
+		unsigned alocatedTextureCount;
 	
 		GLState(void)
 		{
 			resetCache();
-			isTextureRectangle = false;
+			isTextureSRectangle = false;
 			useATIWorkaround = false;
+			alocatedTextureCount = 0;
 		}
 		
 		void resetCache(void)
@@ -122,16 +135,16 @@ namespace GAGCore
 		void checkExtensions(void)
 		{
 			const char *glExtensions = (const char *)glGetString(GL_EXTENSIONS);
-			isTextureRectangle = (strstr(glExtensions, "GL_NV_texture_rectangle") != NULL);
-			isTextureRectangle = isTextureRectangle || (strstr(glExtensions, "GL_EXT_texture_rectangle") != NULL);
-			isTextureRectangle = isTextureRectangle || (strstr(glExtensions, "GL_ARB_texture_rectangle") != NULL);
+			isTextureSRectangle = (strstr(glExtensions, "GL_NV_texture_rectangle") != NULL);
+			isTextureSRectangle = isTextureSRectangle || (strstr(glExtensions, "GL_EXT_texture_rectangle") != NULL);
+			isTextureSRectangle = isTextureSRectangle || (strstr(glExtensions, "GL_ARB_texture_rectangle") != NULL);
 
 			const char *glVendor= (const char *)glGetString(GL_VENDOR);
 			if(strstr(glVendor,"ATI"))
 				useATIWorkaround = true; // ugly temporary bug fix for bug 13823. We think it is an ATI driver bug
 
 			if (verbose)
-				if (isTextureRectangle)
+				if (isTextureSRectangle)
 					std::cout << "Toolkit : GL_NV_texture_rectangle or GL_EXT_texture_rectangle extension present, optimal texture size will be used" << std::endl;
 				else
 					std::cout << "Toolkit : GL_NV_texture_rectangle or GL_EXT_texture_rectangle extension not present, power of two texture will be used" << std::endl;
@@ -155,7 +168,7 @@ namespace GAGCore
 				return;
 		
 			GLenum cap;
-			if (isTextureRectangle)
+			if (isTextureSRectangle)
 				cap = GL_TEXTURE_RECTANGLE_NV;
 			else
 				cap = GL_TEXTURE_2D;
@@ -172,7 +185,7 @@ namespace GAGCore
 			if (_texture == tex)
 				return;
 		
-			if (isTextureRectangle)
+			if (isTextureSRectangle)
 			{
 				if(useATIWorkaround)
 					glBindTexture(GL_TEXTURE_RECTANGLE_NV, 0);
@@ -284,6 +297,7 @@ namespace GAGCore
 		if (_gc->optionFlags & GraphicContext::USEGPU)
 		{
 			glGenTextures(1, reinterpret_cast<GLuint*>(&texture));
+			glState.alocatedTextureCount++;
 			initTextureSize();
 		}
 		#endif
@@ -295,7 +309,7 @@ namespace GAGCore
 		if (_gc->optionFlags & GraphicContext::USEGPU)
 		{
 			// only power of two textures are supported
-			if (!glState.isTextureRectangle)
+			if (!glState.isTextureSRectangle)
 			{
 				// TODO : if anyone has a better way to do it, please tell :-)
 				glState.setTexture(texture);
@@ -342,7 +356,7 @@ namespace GAGCore
 			pixelsPtr = sdlsurface->pixels;
 			pixelFormat = GL_BGRA;
 			#endif
-			if (glState.isTextureRectangle)
+			if (glState.isTextureSRectangle)
 			{
 				glTexImage2D(GL_TEXTURE_RECTANGLE_NV, 0, GL_RGBA, sdlsurface->w, sdlsurface->h, 0, pixelFormat, GL_UNSIGNED_BYTE, pixelsPtr);
 			}
@@ -361,6 +375,7 @@ namespace GAGCore
 		if (_gc->optionFlags & GraphicContext::USEGPU)
 		{
 			glDeleteTextures(1, reinterpret_cast<const GLuint*>(&texture));
+			glState.alocatedTextureCount--;
 			glState.resetCache();
 		}
 		#endif
@@ -1247,8 +1262,26 @@ namespace GAGCore
 		pos = output.find('\r', 0);
 		if(pos != std::string::npos)
 			output = output.substr(0, pos);
-		
+
 		font->drawString(this, x, y, w, output.c_str(), alpha);
+
+		///////////// The following code is for translation textshots ////////////
+		if(!translationPicturesDirectory.empty())
+		{
+			for(std::map<std::string, std::string>::iterator i=texts.begin(); i!=texts.end(); ++i)
+			{
+				if(output.find(i->first)!=std::string::npos)
+				{
+					int width=font->getStringWidth(i->first.c_str());
+					int height=font->getStringHeight(i->first.c_str());
+					int startx=font->getStringWidth(output.substr(0, output.find(i->first)).c_str());
+					drawSquares.push_back(boost::make_tuple(SRectangle(x+startx, y, width, height), i->second, this));
+					wroteTexts.insert(i->first);
+					texts.erase(i);
+					break;
+				}
+			}
+		}
 	}
 	
 	void DrawableSurface::drawString(float x, float y, Font *font, const char *msg, float w, Uint8 alpha)
@@ -1261,8 +1294,26 @@ namespace GAGCore
 		pos = output.find('\r', 0);
 		if(pos != std::string::npos)
 			output = output.substr(0, pos);
-		
+
+		///////////// The following code is for translation textshots ////////////
+		if(!translationPicturesDirectory.empty())
+		{
+			for(std::map<std::string, std::string>::iterator i=texts.begin(); i!=texts.end(); ++i)
+			{
+				if(output.find(i->first)!=std::string::npos)
+				{
+					int width=font->getStringWidth(i->first.c_str());
+					int height=font->getStringHeight(i->first.c_str());
+					int startx=font->getStringWidth(output.substr(0, output.find(i->first)).c_str());
+					drawSquares.push_back(boost::make_tuple(SRectangle(int(x+startx), int(y), width, height), i->second, this));
+					wroteTexts.insert(i->first);
+					texts.erase(i);
+					break;
+				}
+			}
+		}
 		font->drawString(this, x, y, w, output.c_str(), alpha);
+
 	}
 	
 	void DrawableSurface::drawString(int x, int y, Font *font, const std::string &msg, int w, Uint8 alpha)
@@ -1275,6 +1326,24 @@ namespace GAGCore
 		drawString(x, y, font, msg.c_str(), w, alpha);
 	}
 	
+	void DrawableSurface::drawAlphaMap(const std::valarray<float> &map, int mapW, int mapH, int x, int y, int cellW, int cellH, const Color &color)
+	{
+		assert(mapW * mapH <= map.size());
+		
+		for (int dy=0; dy < mapH-1; dy++)
+			for (int dx=0; dx < mapW-1; dx++)
+				drawFilledRect(x + dx * cellW, y + dy * cellH, cellW, cellH, color.applyMultiplyAlpha((Uint8)(255.0f * map[mapW * dy + dx])));
+	}
+	
+	void DrawableSurface::drawAlphaMap(const std::valarray<unsigned char> &map, int mapW, int mapH, int x, int y, int cellW, int cellH, const Color &color)
+	{
+		assert(mapW * mapH <= map.size());
+		
+		for (int dy=0; dy < mapH-1; dy++)
+			for (int dx=0; dx < mapW-1; dx++)
+				drawFilledRect(x + dx * cellW, y + dy * cellH, cellW, cellH, color.applyMultiplyAlpha(map[mapW * dy + dx]));
+	}
+	
 	// compat
 	void DrawableSurface::drawString(int x, int y, Font *font, int i)
 	{
@@ -1283,8 +1352,55 @@ namespace GAGCore
 		this->drawString(x, y, font, str.str());
 	}
 	
+	//This code is for the textshot code
+	std::map<std::string, std::string> DrawableSurface::texts;
+	std::set<std::string> DrawableSurface::wroteTexts;
+	std::vector<boost::tuple<DrawableSurface::SRectangle, std::string, GAGCore::DrawableSurface*> > DrawableSurface::drawSquares;
+	std::string DrawableSurface::translationPicturesDirectory;
+
+	void DrawableSurface::flushTextPictures()
+	{
+		using namespace GAGCore;
+		for(std::vector<boost::tuple<SRectangle, std::string, DrawableSurface*> >::iterator i=drawSquares.begin(); i!=drawSquares.end();)
+		{
+			DrawableSurface toPrint(i->get<2>()->getW(), i->get<2>()->getH());
+			glFlush();
+			toPrint.drawSurface(0, 0, i->get<2>());
+			int x=i->get<0>().x;
+			int y=i->get<0>().y;
+			int width=i->get<0>().w;
+			int height=i->get<0>().h;
+
+			toPrint.drawRect(x-2, y-2, width+4, height+4, Color(255, 126, 21));
+			toPrint.drawRect(x-3, y-3, width+6, height+6, Color(255, 126, 21));
+			toPrint.drawCircle(x+width/2, y+height/2, std::max(width+4, height+4)/2+4, Color(255, 126, 21));
+			toPrint.drawCircle(x+width/2, y+height/2, std::max(width+4, height+4)/2+5, Color(255, 126, 21));
+			toPrint.drawCircle(x+width/2, y+height/2, std::max(width+4, height+4)/2+6, Color(255, 126, 21));
+
+			// Print it using virtual filesystem
+			for (size_t i2 = 0; i2 < Toolkit::getFileManager()->getDirCount(); i2++)
+			{
+				std::string fullFileName = translationPicturesDirectory + DIR_SEPARATOR_S + "text-" + i->get<1>();
+				if (SDL_SaveBMP(toPrint.sdlsurface, (fullFileName+".bmp").c_str()) == 0)
+				{
+					break;
+				}
+			}
+			i=drawSquares.erase(i);
+		}
+	}
+
+	void DrawableSurface::printFinishingText()
+	{
+		if(!texts.empty())
+			std::cout<<"The following requested translation texts where never drawn to the screen, or too mangled to be detected:"<<std::endl;
+		for(std::map<std::string, std::string>::iterator i=texts.begin(); i!=texts.end(); ++i)
+		{
+			std::cout<<"\t"<<i->second<<std::endl;
+		}
+	}
+
 	// here begin the Graphic Context part
-	
 	void GraphicContext::setClipRect(int x, int y, int w, int h)
 	{
 		DrawableSurface::setClipRect(x, y, w, h);
@@ -1570,7 +1686,65 @@ namespace GAGCore
 			DrawableSurface::drawSurface(static_cast<int>(x), static_cast<int>(y), static_cast<int>(w), static_cast<int>(h), surface, sx, sy, sw, sh, alpha);
 	}
 	
-	// compat... this is there because it sems gc is not able to do function overloading with several levels of inheritance
+	void GraphicContext::drawAlphaMap(const std::valarray<float> &map, int mapW, int mapH, int x, int y, int cellW, int cellH, const Color &color)
+	{
+		#ifdef HAVE_OPENGL
+		if (_gc->optionFlags & GraphicContext::USEGPU)
+		{
+			assert(mapW * mapH <= map.size());
+			
+			float fr = 255.0f*(float)color.r;
+			float fg = 255.0f*(float)color.g;
+			float fb = 255.0f*(float)color.b;
+			
+			glState.doBlend(1);
+			glState.doTexture(0);
+			for (int dy=0; dy < mapH-1; dy++)
+			{
+				glBegin(GL_TRIANGLE_STRIP);
+				for (int dx=0; dx < mapW; dx++)
+				{
+					glColor4f(fr, fg, fb, map[mapW * dy + dx]);
+					glVertex2f(x + dx * cellW, y + dy * cellH);
+					glColor4f(fr, fg, fb, map[mapW * (dy + 1) + dx]);
+					glVertex2f(x + dx * cellW, y + (dy + 1) * cellH);
+				}
+				glEnd();
+			}
+		}
+		else
+		#endif
+			DrawableSurface::drawAlphaMap(map, mapW, mapH, x, y, cellW, cellH, color);
+	}
+	
+	void GraphicContext::drawAlphaMap(const std::valarray<unsigned char> &map, int mapW, int mapH, int x, int y, int cellW, int cellH, const Color &color)
+	{
+		#ifdef HAVE_OPENGL
+		if (_gc->optionFlags & GraphicContext::USEGPU)
+		{
+			assert(mapW * mapH <= map.size());
+			
+			glState.doBlend(1);
+			glState.doTexture(0);
+			for (int dy=0; dy < mapH-1; dy++)
+			{
+				glBegin(GL_TRIANGLE_STRIP);
+				for (int dx=0; dx < mapW; dx++)
+				{
+					glColor4ub(color.r, color.g, color.b, map[mapW * dy + dx]);
+					glVertex2f(x + dx * cellW, y + dy * cellH);
+					glColor4ub(color.r, color.g, color.b, map[mapW * (dy + 1) + dx]);
+					glVertex2f(x + dx * cellW, y + (dy + 1) * cellH);
+				}
+				glEnd();
+			}
+		}
+		else
+		#endif
+			DrawableSurface::drawAlphaMap(map, mapW, mapH, x, y, cellW, cellH, color);
+	}
+	
+	// compat... this is there because it sems gcc is not able to do function overloading with several levels of inheritance
 	void GraphicContext::drawPixel(int x, int y, Uint8 r, Uint8 g, Uint8 b, Uint8 a)
 	{
 		drawPixel(x, y, Color(r, g, b, a));
@@ -1811,6 +1985,7 @@ namespace GAGCore
 	
 	void GraphicContext::nextFrame(void)
 	{
+		DrawableSurface::nextFrame();
 		if (sdlsurface)
 		{
 			if (optionFlags & CUSTOMCURSOR)
@@ -1824,10 +1999,15 @@ namespace GAGCore
 			
 			#ifdef HAVE_OPENGL
 			if (optionFlags & GraphicContext::USEGPU)
+			{
 				SDL_GL_SwapBuffers();
+				//fprintf(stderr, "%d allocated GPU textures\n", glState.alocatedTextureCount);
+			}
 			else
 			#endif
+			{
 				SDL_Flip(sdlsurface);
+			}
 		}
 	}
 	
