@@ -1015,17 +1015,17 @@ Order *AICastor::controlUpgrades()
 		if (b->type->type == "defencetower")
 		{
 			if (b->hp*4<b->type->hpMax*1)
-				return new OrderConstruction(b->gid);
+				return new OrderConstruction(b->gid, 1, 1);
 		}
 		else if (b->type->maxUnitInside)
 		{
 			if (b->hp*4<b->type->hpMax*3)
-				return new OrderConstruction(b->gid);
+				return new OrderConstruction(b->gid, 1, 1);
 		}
 		else
 		{
 			if (b->hp*4<b->type->hpMax*2)
-				return new OrderConstruction(b->gid);
+				return new OrderConstruction(b->gid, 1, 1);
 		}
 	}
 	// Do we want to upgrade it:
@@ -1078,7 +1078,7 @@ Order *AICastor::controlUpgrades()
 		}
 	}
 	controlUpgradeDelay=32;
-	return new OrderConstruction(b->gid);
+	return new OrderConstruction(b->gid, 1, 1);
 }
 
 
@@ -1215,7 +1215,7 @@ Order *AICastor::controlStrikes()
 		{
 			Sint32 typeNum=globalContainer->buildingsTypes.getTypeNum("warflag", 0, false);
 			fprintf(logFile,  " create\n");
-			return new OrderCreate(team->teamNumber, x, y, typeNum);
+			return new OrderCreate(team->teamNumber, x, y, typeNum, 1, 1);
 		}
 		else
 		{
@@ -2915,7 +2915,7 @@ Order *AICastor::findGoodBuilding(Sint32 typeNum, bool food, bool defense, bool
 		
 		Sint32 x=(bestIndex&map->wMask);
 		Sint32 y=((bestIndex>>map->wDec)&map->hMask);
-		return new OrderCreate(team->teamNumber, x, y, typeNum);
+		return new OrderCreate(team->teamNumber, x, y, typeNum, 1, 1);
 	}
 	
 	return NULL;
