@@ -878,7 +878,7 @@ Order *AINumbi::mayAttack(int critticalMass, int critticalTimeout, Sint32 number
 		{
 			Sint32 typeNum=globalContainer->buildingsTypes.getTypeNum("warflag", 0, false);
 			//printf("AI: OrderCreateWarFlag(%d, %d)\n", ex, ey);
-			return new OrderCreate(teamNumber, ex, ey, typeNum);
+			return new OrderCreate(teamNumber, ex, ey, typeNum, 1, 1);
 		}
 		else
 			return new NullOrder();
@@ -939,7 +939,7 @@ Order *AINumbi::adjustBuildings(const int numbers, const int numbersInc, const i
 		{
 			Sint32 typeNum=globalContainer->buildingsTypes.getTypeNum(IntBuildingType::typeFromShortNumber(buildingType), 0, true);
 			int teamNumber=team->teamNumber;
-			return new OrderCreate(teamNumber, x, y, typeNum);
+			return new OrderCreate(teamNumber, x, y, typeNum, 1, 1);
 		}
 		//printf("AI: findNewEmplacement(%d) failed.\n", buildingType);
 		return new NullOrder();
@@ -972,7 +972,7 @@ Order *AINumbi::checkoutExpands(const int numbers, const int workers)
 		{
 			Sint32 typeNum=globalContainer->buildingsTypes.getTypeNum("swarm", 0, true);
 			int teamNumber=team->teamNumber;
-			return new OrderCreate(teamNumber, x, y, typeNum);
+			return new OrderCreate(teamNumber, x, y, typeNum, 1, 1);
 		}
 		return new NullOrder();
 	}
@@ -1097,31 +1097,31 @@ Order *AINumbi::mayUpgrade(const int ptrigger, const int ntrigger)
 		{
 			Building *b=foodBuilding[0];
 			if (b)
-				return new OrderConstruction(b->gid);
+				return new OrderConstruction(b->gid, 1, 1);
 		}
 		if (numberHealth[0]>numberUpgradingHealth[1])
 		{
 			Building *b=healthBuilding[0];
 			if (b)
-				return new OrderConstruction(b->gid);
+				return new OrderConstruction(b->gid, 1, 1);
 		}
 		if (numberAttack[0]>numberUpgradingAttack[1])
 		{
 			Building *b=attackBuilding[0];
 			if (b)
-				return new OrderConstruction(b->gid);
+				return new OrderConstruction(b->gid, 1, 1);
 		}
 		if (numberScience[0]>numberUpgradingScience[1]+1)
 		{
 			Building *b=scienceBuilding[0];
 			if (b)
-				return new OrderConstruction(b->gid);
+				return new OrderConstruction(b->gid, 1, 1);
 		}
 		if (numberDefense[0]>numberUpgradingDefense[1])
 		{
 			Building *b=defenseBuilding[0];
 			if (b)
-				return new OrderConstruction(b->gid);
+				return new OrderConstruction(b->gid, 1, 1);
 		}
 	}
 	
@@ -1134,31 +1134,31 @@ Order *AINumbi::mayUpgrade(const int ptrigger, const int ntrigger)
 		{
 			Building *b=foodBuilding[0];
 			if (b)
-				return new OrderConstruction(b->gid);
+				return new OrderConstruction(b->gid, 1, 1);
 		}
 		if (numberHealth[1]>numberUpgradingHealth[2])
 		{
 			Building *b=healthBuilding[0];
 			if (b)
-				return new OrderConstruction(b->gid);
+				return new OrderConstruction(b->gid, 1, 1);
 		}
 		if (numberAttack[1]>numberUpgradingAttack[2])
 		{
 			Building *b=attackBuilding[0];
 			if (b)
-				return new OrderConstruction(b->gid);
+				return new OrderConstruction(b->gid, 1, 1);
 		}
 		if (numberScience[1]>numberUpgradingScience[2]+1)
 		{
 			Building *b=scienceBuilding[0];
 			if (b)
-				return new OrderConstruction(b->gid);
+				return new OrderConstruction(b->gid, 1, 1);
 		}
 		if (numberDefense[1]>numberUpgradingDefense[2])
 		{
 			Building *b=defenseBuilding[0];
 			if (b)
-				return new OrderConstruction(b->gid);
+				return new OrderConstruction(b->gid, 1, 1);
 		}
 	}
 	
