@@ -32,12 +32,12 @@ using namespace GAGCore;
 CampaignScreen::CampaignScreen(const std::string &text)
 {
 	
-	addWidget(new TextButton(20, 20, 70, 25, ALIGN_RIGHT, ALIGN_BOTTOM, "", -1, -1, "standard", Toolkit::getStringTable()->getString("[Start]"), 0, '\r'));
+	addWidget(new TextButton(20, 20, 180, 40, ALIGN_LEFT, ALIGN_BOTTOM, "", -1, -1, "menu", Toolkit::getStringTable()->getString("[Start]"), 0, '\r'));
 
-	addWidget(new TextButton(20, 20, 70, 25, ALIGN_LEFT, ALIGN_BOTTOM, "", -1, -1, "standard", Toolkit::getStringTable()->getString("[Cancel]"), 1, 27));
+	addWidget(new TextButton(20, 20, 180, 40, ALIGN_RIGHT, ALIGN_BOTTOM, "", -1, -1, "menu", Toolkit::getStringTable()->getString("[Cancel]"), 1, 27));
 	
 	
-	addWidget(new TextArea(20, 20, 20, 60, ALIGN_FILL, ALIGN_FILL, "standard", true, text.c_str()));
+	addWidget(new TextArea(20, 20, 20, 80, ALIGN_FILL, ALIGN_FILL, "standard", true, text.c_str()));
 }
 
 void CampaignScreen::onAction(Widget *source, Action action, int par1, int par2)
