@@ -21,6 +21,7 @@
 #include "GUIGlob2FileList.h"
 #include "GUIMapPreview.h"
 #include "Session.h"
+#include "GlobalContainer.h"
 #include <FormatableString.h>
 #include <GUIButton.h>
 #include <GUIText.h>
@@ -51,12 +52,12 @@ ChooseMapScreen::ChooseMapScreen(const char *directory, const char *extension, b
 	else if (strcmp(directory, "games") == 0)
 	{
 		title = new Text(0, 18, ALIGN_FILL, ALIGN_SCREEN_CENTERED, "menu", Toolkit::getStringTable()->getString("[choose game]"));
-		deleteMap = new TextButton(220, 430, 200, 30, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "", -1, -1, "standard", Toolkit::getStringTable()->getString("[Delete game]"), DELETEGAME);
+		deleteMap = new TextButton(220, 430, 200, 20, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "", -1, -1, "standard", Toolkit::getStringTable()->getString("[Delete game]"), DELETEGAME);
 		addWidget(deleteMap);
 	}
 	else
 	{
-		title = new Text(0, 18, ALIGN_FILL, ALIGN_SCREEN_CENTERED, "menu", Toolkit::getStringTable()->getString("[choose campaign]"));
+		title = new Text(0, 18, ALIGN_FILL, ALIGN_SCREEN_CENTERED, "menu", Toolkit::getStringTable()->getString("[choose game]"));
 	}
 	addWidget(title);
 	mapName=new Text(440, 60+128+25, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", "", 180);
@@ -70,6 +71,34 @@ ChooseMapScreen::ChooseMapScreen(const char *directory, const char *extension, b
 	mapDate=new Text(440, 60+128+125, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", "", 180);
 	addWidget(mapDate);
 	
+	globalContainer->settings.tempVarPrestige = 3000;
+	prestigeRatio=new Number(20, 20, 100, 18, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, 18, "menu");
+	prestigeRatio->add(0);
+	prestigeRatio->add(0);
+	prestigeRatio->add(100);
+	prestigeRatio->add(200);
+	prestigeRatio->add(300);
+	prestigeRatio->add(400);
+	prestigeRatio->add(500);
+	prestigeRatio->add(600);
+	prestigeRatio->add(700);
+	prestigeRatio->add(800);
+	prestigeRatio->add(900);
+	prestigeRatio->add(1000);
+	prestigeRatio->add(1100);
+	prestigeRatio->add(1200);
+	prestigeRatio->add(1300);
+	prestigeRatio->add(1400);
+	prestigeRatio->add(1500);
+	prestigeRatio->add(1600);
+	prestigeRatio->add(1700);
+	prestigeRatio->add(1800);
+	prestigeRatio->add(1900);
+	prestigeRatio->add(2000);
+	prestigeRatio->setNth(1);
+	prestigeRatio->visible=false;
+	addWidget(prestigeRatio);
+	
 	validMapSelected = false;
 }
 
@@ -115,6 +144,14 @@ void ChooseMapScreen::onAction(Widget *source, Action action, int par1, int par2
 		}
 		delete stream;
 	}
+	else if (action==NUMBER_ELEMENT_SELECTED)
+	{
+		if (prestigeRatio->getNth() == 0)
+		{
+			prestigeRatio->setNth(1);
+		}
+		globalContainer->settings.tempVarPrestige=(prestigeRatio->getNth() - 1) * 100;
+	}
 	else if ((action == BUTTON_RELEASED) || (action == BUTTON_SHORTCUT))
 	{
 		if (source == ok)
