@@ -105,16 +105,6 @@ Engine::~Engine()
 	}
 }
 
-int Engine::initCampaign()
-{
-	ChooseMapScreen loadCampaignScreen("campaigns", "map", true);;
-	int lgs = loadCampaignScreen.execute(globalContainer->gfx, 40);
-	if (lgs == ChooseMapScreen::CANCEL)
-		return EE_CANCEL;
-
-	return initCampaign(glob2NameToFilename("campaigns", loadCampaignScreen.sessionInfo.getMapNameC(), "map"));
-}
-
 int Engine::initCampaign(const std::string &mapName)
 {
 	if (!loadGame(mapName))
@@ -180,6 +170,13 @@ int Engine::initCampaign(const std::string &mapName)
 	return EE_NO_ERROR;
 }
 
+int Engine::initCampaign(const std::string &mapName, Campaign& campaign, const std::string& missionName)
+{
+	int end=initCampaign(mapName);
+	gui.setCampaignGame(campaign, missionName);
+	return end;
+}
+
 int Engine::initCustom(void)
 {
 	CustomGameScreen customGameScreen;
@@ -451,7 +448,10 @@ int Engine::run(void)
 					// we get and push ai orders
 					for (int i=0; i<gui.game.session.numberOfPlayer; i++)
 						if (gui.game.players[i]->ai)
-							net->pushOrder(gui.game.players[i]->ai->getOrder(gui.gamePaused), i);
+						{
+							Order* order=gui.game.players[i]->ai->getOrder(gui.gamePaused);
+							net->pushOrder(order, i);
+						}
 					
 					ticksToDelayInside=net->ticksToDelayInside();
 					ticksDelayedInside=ticksToDelayInside+computationAvailableTicks/2;
@@ -565,15 +565,6 @@ int Engine::run(void)
 				doRunOnceAgain=true;
 			gui.toLoadGameFileName[0]=0; // Avoid the communication system between GameGUI and Engine to loop.
 		}
-		else if (gui.game.nextMap.length() > 0)
-		{
-			// if we have won, managed to load next map, we do it again. Only campaigns can be linked, so we look into the campaigns subdirectory
-			if (gui.game.isGameEnded && gui.getLocalTeam()->isAlive)
-			{
-				std::string filename = std::string("campaigns/") +  gui.game.nextMap;
-				doRunOnceAgain = (initCampaign(filename.c_str()) == EE_NO_ERROR);
-			}
-		}
 	}
 	
 	if (globalContainer->runNoX)
