@@ -43,8 +43,10 @@
 #include "Unit.h"
 #include "UnitSkin.h"
 #include "Utilities.h"
+#include "GameGUI.h"
 
 #include "Brush.h"
+#include "DynamicClouds.h"
 
 #define BULLET_IMGID 0
 
@@ -98,9 +100,14 @@ Game::~Game()
 }
 
 void Game::init(GameGUI *gui)
-{
+{	
 	this->gui=gui;
 	buildProjects.clear();
+
+    //If the GameGUI provided is not NULL, meaning that this is a real game not a
+	//map edit, that initialize the preset numbers of units to assign to buildings
+	if(gui)
+		gui->initUnitCount();
 	
 	if (globalContainer->runNoX)
 		minimap=NULL;
@@ -207,7 +214,7 @@ void Game::executeOrder(Order *order, int localPlayer)
 			bool isRoom=checkRoomForBuilding(posX, posY, bt, oc->teamNumber);
 			if (isVirtual || isRoom)
 			{
-				Building *b=addBuilding(posX, posY, oc->typeNum, oc->teamNumber);
+				Building *b=addBuilding(posX, posY, oc->typeNum, oc->teamNumber, oc->unitWorking, oc->unitWorkingFuture);
 				if (b)
 				{
 					fprintf(logFile, "ORDER_CREATE (%d, %d, %d)", posX, posY, bt->shortTypeNum);
@@ -218,11 +225,13 @@ void Game::executeOrder(Order *order, int localPlayer)
 			else if (!isVirtual && !isRoom && map.isHardSpaceForBuilding(posX, posY, w, h))
 			{
 				BuildProject buildProject;
-				buildProject.posX=posX;
-				buildProject.posY=posY;
+				buildProject.posX = posX;
+				buildProject.posY = posY;
 				fprintf(logFile, "new BuildProject (%d, %d)", posX, posY);
-				buildProject.teamNumber=oc->teamNumber;
-				buildProject.typeNum=oc->typeNum;
+				buildProject.teamNumber = oc->teamNumber;
+				buildProject.typeNum = oc->typeNum;
+				buildProject.unitWorking = oc->unitWorking;
+				buildProject.unitWorkingFuture = oc->unitWorkingFuture;
 				buildProjects.push_back(buildProject);
 				Uint32 teamMask=Team::teamNumberToMask(oc->teamNumber);
 				for (int y=posY; y<posY+h; y++)
@@ -309,11 +318,6 @@ void Game::executeOrder(Order *order, int localPlayer)
 				}
 				else
 				{
-					if (b->type->zonable[WORKER])
-					{
-						b->updateClearingFlag(0);
-						b->updateClearingFlag(1);
-					}
 					for (int i=0; i<2; i++)
 					{
 						b->dirtyLocalGradient[i]=true;
@@ -351,8 +355,6 @@ void Game::executeOrder(Order *order, int localPlayer)
 				memcpy(b->clearingRessources, omcf->clearingRessources, sizeof(bool)*BASIC_COUNT);
 				if (order->sender!=localPlayer)
 					memcpy(b->clearingRessourcesLocal, omcf->clearingRessources, sizeof(bool)*BASIC_COUNT);
-				b->updateClearingFlag(0);
-				b->updateClearingFlag(1);
 			}
 		}
 		break;
@@ -413,11 +415,6 @@ void Game::executeOrder(Order *order, int localPlayer)
 				}
 				else
 				{
-					if (drop && b->type->zonable[WORKER])
-					{
-						b->updateClearingFlag(0);
-						b->updateClearingFlag(1);
-					}
 					for (int i=0; i<2; i++)
 					{
 						b->dirtyLocalGradient[i]=true;
@@ -647,7 +644,9 @@ void Game::executeOrder(Order *order, int localPlayer)
 		{
 			if (!isPlayerAlive)
 				break;
-			Uint16 gid=((OrderConstruction *)order)->gid;
+			OrderConstruction *oc = ((OrderConstruction *)order);
+			Uint16 gid = oc->gid;
+			
 			int team=Building::GIDtoTeam(gid);
 			int id=Building::GIDtoID(gid);
 			Team *t=teams[team];
@@ -655,7 +654,7 @@ void Game::executeOrder(Order *order, int localPlayer)
 			if (b)
 			{
 				fprintf(logFile, "ORDER_CONSTRUCTION");
-				b->launchConstruction();
+				b->launchConstruction(oc->unitWorking, oc->unitWorkingFuture);
 			}
 		}
 		break;
@@ -932,7 +931,10 @@ bool Game::load(GAGCore::InputStream *stream)
 
 	// we have to finish Team's loading:
 	for (int i=0; i<session.numberOfTeam; i++)
+	{
+		teams[i]->stats.setMapSize(map.getW(), map.getH());
 		teams[i]->update();
+	}
 	
 	for (int i=0; i<session.numberOfTeam; i++)
 		teams[i]->integrity();
@@ -948,18 +950,27 @@ bool Game::load(GAGCore::InputStream *stream)
 	
 	if (session.versionMinor < 37)
 	{
-		nextMap = "";
 		campaignText = "";
 	}
 	else
 	{
-		nextMap = stream->readText("nextMap");
+		if(session.versionMinor < 55)
+			stream->readText("nextMap");
 		campaignText = stream->readText("campaignText");
 	}
-
-	// Compute new max prestige
-	prestigeToReach = std::max(MIN_MAX_PRESIGE, session.numberOfTeam*TEAM_MAX_PRESTIGE);
-
+//game reverts to default prestige settings if higher then maximum (2000)
+//this is done to maximize efficiency if maximum settings are ever changed this 
+//will need to be changed as well 
+	if (session.varPrestige > 2000)
+	{	
+		// default prestige calculation
+		prestigeToReach = std::max(MIN_MAX_PRESIGE, session.numberOfTeam*TEAM_MAX_PRESTIGE);
+	}
+	else
+	{
+	// custom prestige
+		prestigeToReach = session.varPrestige;
+	}
 	stream->readLeaveSection();
 	return true;
 }
@@ -1029,14 +1040,13 @@ void Game::save(GAGCore::OutputStream *stream, bool fileIsAMap, const char* name
 	SAVE_OFFSET(stream, 32, "mapScriptOffset");
 	script.save(stream, this);
 	
-	stream->writeText(nextMap, "nextMap");
 	stream->writeText(campaignText, "campaignText");
 	stream->writeLeaveSection();
 }
 
 void Game::buildProjectSyncStep(Sint32 localTeam)
 {
-	for (std::list<BuildProject>::iterator bpi=buildProjects.begin(); bpi!=buildProjects.end(); bpi++)
+	for (std::list<BuildProject>::iterator bpi=buildProjects.begin(); bpi!=buildProjects.end();)
 	{
 		int posX=bpi->posX&map.getMaskW();
 		int posY=bpi->posY&map.getMaskH();
@@ -1060,12 +1070,14 @@ void Game::buildProjectSyncStep(Sint32 localTeam)
 						map.localForbiddenMap.set(index, false);
 				}
 			map.updateForbiddenGradient(teamNumber);
-			buildProjects.erase(bpi);
-			break;
+			std::list<BuildProject>::iterator to_erase=bpi;
+			bpi++;
+			buildProjects.erase(to_erase);
+			continue;
 		}
-		if (checkRoomForBuilding(posX, posY, bt, teamNumber))
+		else if (checkRoomForBuilding(posX, posY, bt, teamNumber))
 		{
-			Building *b=addBuilding(posX, posY, typeNum, teamNumber);
+			Building *b=addBuilding(posX, posY, typeNum, teamNumber, bpi->unitWorking, bpi->unitWorkingFuture);
 			if (b)
 			{
 				Uint32 notTeamMask=~Team::teamNumberToMask(teamNumber);
@@ -1083,40 +1095,47 @@ void Game::buildProjectSyncStep(Sint32 localTeam)
 				b->owner->addToStaticAbilitiesLists(b);
 				b->update();
 				fprintf(logFile, "BuildProject success (%d, %d)\n", posX, posY);
-				buildProjects.erase(bpi);
-				break;
+				std::list<BuildProject>::iterator to_erase=bpi;
+				bpi++;
+				buildProjects.erase(to_erase);
+				continue;
 			}
 		}
+		bpi++;
 	}
 }
 
 void Game::wonSyncStep(void)
 {
-	totalPrestige=0;
-	isGameEnded=false;
-	int greatestPrestige=0;
-	
-	for (int i=0; i<session.numberOfTeam; i++)
+// prestige of 0 results in infinite prestige
+	if (session.varPrestige > 0)
 	{
-		bool isOtherAlive=false;
-		for (int j=0; j<session.numberOfTeam; j++)
-		{
-			if ((j!=i) && (!( ((teams[i]->me) & (teams[j]->allies)) /*&& ((teams[j]->me) & (teams[i]->allies))*/ )) && (teams[j]->isAlive))
-				isOtherAlive=true;
-		}
-		teams[i]->hasWon |= !isOtherAlive;
-		isGameEnded |= teams[i]->hasWon;
-		totalPrestige += teams[i]->prestige;
-		if (greatestPrestige < teams[i]->prestige) greatestPrestige = teams[i]->prestige;
-	}
-
-	if (totalPrestige >= prestigeToReach)
-	{
-		totalPrestigeReached=true;
-		isGameEnded=true;
-
+		totalPrestige=0;
+		isGameEnded=false;
+		int greatestPrestige=0;
+		
 		for (int i=0; i<session.numberOfTeam; i++)
-			teams[i]->hasWon = teams[i]->prestige == greatestPrestige;
+		{
+			bool isOtherAlive=false;
+			for (int j=0; j<session.numberOfTeam; j++)
+			{
+				if ((j!=i) && (!( ((teams[i]->me) & (teams[j]->allies)) /*&& ((teams[j]->me) & (teams[i]->allies))*/ )) && (teams[j]->isAlive))
+					isOtherAlive=true;
+			}
+			teams[i]->hasWon |= !isOtherAlive;
+			isGameEnded |= teams[i]->hasWon;
+			totalPrestige += teams[i]->prestige;
+			if (greatestPrestige < teams[i]->prestige) greatestPrestige = teams[i]->prestige;
+		}
+	
+		if (totalPrestige >= prestigeToReach)
+		{
+			totalPrestigeReached=true;
+			isGameEnded=true;
+	
+			for (int i=0; i<session.numberOfTeam; i++)
+				teams[i]->hasWon = teams[i]->prestige == greatestPrestige;
+		}
 	}
 }
 
@@ -1178,7 +1197,9 @@ void Game::syncStep(Sint32 localTeam)
 		}
 		
 		if (!globalContainer->runNoX)
+		{
 			renderMiniMap(localTeam, false, stepCounter%25, 25);
+		}
 
 		if ((stepCounter&15)==1)
 			buildProjectSyncStep(localTeam);
@@ -1324,7 +1345,7 @@ Unit *Game::addUnit(int x, int y, int team, Sint32 typeNum, int level, int delta
 	return teams[team]->myUnits[id];
 }
 
-Building *Game::addBuilding(int x, int y, int typeNum, int teamNumber)
+Building *Game::addBuilding(int x, int y, int typeNum, int teamNumber, Sint32 unitWorking, Sint32 unitWorkingFuture)
 {
 	Team *team=teams[teamNumber];
 	assert(team);
@@ -1345,7 +1366,7 @@ Building *Game::addBuilding(int x, int y, int typeNum, int teamNumber)
 	int w=globalContainer->buildingsTypes.get(typeNum)->width;
 	int h=globalContainer->buildingsTypes.get(typeNum)->height;
 
-	Building *b=new Building(x&map.getMaskW(), y&map.getMaskH(), gid, typeNum, team, &globalContainer->buildingsTypes);
+	Building *b=new Building(x&map.getMaskW(), y&map.getMaskH(), gid, typeNum, team, &globalContainer->buildingsTypes, unitWorking, unitWorkingFuture);
 
 	if (b->type->canExchange)
 		team->canExchange.push_front(b);
@@ -1501,6 +1522,17 @@ bool Game::checkHardRoomForBuilding(int x, int y, const BuildingType *bt)
 	return map.isHardSpaceForBuilding(x, y, w, h);
 }
 
+
+
+Unit* Game::getUnit(int guid)
+{
+	if(guid == NOGUID)
+		return NULL;
+	return teams[Unit::GIDtoTeam(guid)]->myUnits[Unit::GIDtoID(guid)];
+}
+
+
+
 void Game::drawPointBar(int x, int y, BarOrientation orientation, int maxLength, int actLength, Uint8 r, Uint8 g, Uint8 b, int barWidth)
 {
 	assert(maxLength>=0);
@@ -1726,29 +1758,17 @@ struct BuildingPosComp
 	}
 };
 
-void Game::drawMap(int sx, int sy, int sw, int sh, int viewportX, int viewportY, int localTeam, Uint32 drawOptions)
+inline void Game::drawMapWater(int sw, int sh, int viewportX, int viewportY, int time)
 {
-	static int cloudDisplacement = -1023;
-	int id;
-	int left=(sx>>5);
-	int top=(sy>>5);
-	int right=((sx+sw+31)>>5);
-	int bot=((sy+sh+31)>>5);
-	std::set<Building *, BuildingPosComp> buildingList;
-
-	// compute cloud coordinates, used for displacement of clouds but also water
-	if (cloudDisplacement > 0)
-		cloudDisplacement -= 1024;
-	else
-		cloudDisplacement++;
-	
-	// we draw water
-	int waterStartX = -((viewportX<<5) % 512);
+	int waterStartX = -(((viewportX<<5)+time/2) % 512);
 	int waterStartY = -((viewportY<<5) % 512);
 	for (int y=waterStartY; y<sh; y += 512)
-		for (int x=waterStartX + (cloudDisplacement / 2); x<sw; x += 512)
+		for (int x=waterStartX; x<sw; x += 512)
 			globalContainer->gfx->drawSprite(x, y, globalContainer->terrainWater, 0);
-	
+}
+
+inline void Game::drawMapTerrain(int left, int top, int right, int bot, int viewportX, int viewportY, int localTeam, Uint32 drawOptions)
+{
 	// we draw the terrains, eventually with debug rects:
 	for (int y=top; y<=bot; y++)
 		for (int x=left; x<=right; x++)
@@ -1765,7 +1785,7 @@ void Game::drawMap(int sx, int sy, int sw, int sh, int viewportX, int viewportY,
 				((drawOptions & DRAW_WHOLE_MAP) != 0))
 			{
 				// draw terrain
-				id=map.getTerrain(x+viewportX, y+viewportY);
+				int id=map.getTerrain(x+viewportX, y+viewportY);
 				Sprite *sprite;
 				if (id<272)
 				{
@@ -1780,7 +1800,10 @@ void Game::drawMap(int sx, int sy, int sw, int sh, int viewportX, int viewportY,
 				if ((id < 256) || (id >= 256+16))
 					globalContainer->gfx->drawSprite(x<<5, y<<5, sprite, id);
 			}
+}
 
+inline void Game::drawMapRessources(int left, int top, int right, int bot, int viewportX, int viewportY, int localTeam, Uint32 drawOptions)
+{
 	for (int y=top; y<=bot; y++)
 		for (int x=left; x<=right; x++)
 			if (
@@ -1795,7 +1818,6 @@ void Game::drawMap(int sx, int sy, int sw, int sh, int viewportX, int viewportY,
 				(map.isMapDiscovered(x+viewportX+1, y+viewportY+1,  teams[localTeam]->me)) ||
 				((drawOptions & DRAW_WHOLE_MAP) != 0))
 			{
-				// draw ressource
 				Ressource r=map.getRessource(x+viewportX, y+viewportY);
 				if (r.type!=NO_RES_TYPE)
 				{
@@ -1818,19 +1840,11 @@ void Game::drawMap(int sx, int sy, int sw, int sh, int viewportX, int viewportY,
 					globalContainer->gfx->drawSprite((x<<5)-dx, (y<<5)-dy, sprite, imgid);
 				}
 			}
+}
 
-	//for (int y=top; y<=bot; y++)
-	//	for (int x=left; x<=right; x++)
-	//		if (map.getBuilding(x+viewportX, y+viewportY)!=NOGBID)
-	//			globalContainer->gfx->drawRect(x<<5, y<<5, 32, 32, 255, 0, 0);
-	
-	//for (int y=top; y<=bot; y++)
-	//	for (int x=left; x<=right; x++)
-	//		if (!map.isFreeForGroundUnit(x+viewportX, y+viewportY, 0, 0))
-	//			globalContainer->gfx->drawRect(x<<5, y<<5, 32, 32, 255, 0, 0);
-	
-	// We draw ground units:
-	mouseUnit=NULL;
+inline void Game::drawMapGroundUnits(int left, int top, int right, int bot, int sw, int sh, int viewportX, int viewportY, int localTeam, Uint32 drawOptions)
+{
+	mouseUnit=NULL;//??
 	for (int y=top-1; y<=bot; y++)
 		for (int x=left-1; x<=right; x++)
 		{
@@ -1838,19 +1852,18 @@ void Game::drawMap(int sx, int sy, int sw, int sh, int viewportX, int viewportY,
 			if (gid!=NOGUID)
 				drawUnit(x, y, gid, viewportX, viewportY, (sw>>5), (sh>>5), localTeam, drawOptions);
 		}
-	
-	// We draw debug area:
+}
+
+inline void Game::drawMapDebugAreas(int left, int top, int right, int bot, int sw, int sh, int viewportX, int viewportY, int localTeam, Uint32 drawOptions)
+{
 	if (false)
 		for (int y=top-1; y<=bot; y++)
 			for (int x=left-1; x<=right; x++)
 			{
 				//globalContainer->gfx->drawString((x<<5), (y<<5), globalContainer->littleFont, ((AICastor *)players[1]->ai->aiImplementation)->wheatCareMap[0][(x+viewportX)+(y+viewportY)*map.w]);
-				
 				//globalContainer->gfx->drawString((x<<5), (y<<5), globalContainer->littleFont, ((AICastor *)players[1]->ai->aiImplementation)->notGrassMap[(x+viewportX)+(y+viewportY)*map.w]);
-				
 //				globalContainer->gfx->drawString((x<<5), (y<<5), globalContainer->littleFont, map.getExplored(x+viewportX, y+viewportY, 0));
-
-				globalContainer->gfx->drawString((x<<5), (y<<5), globalContainer->littleFont, ((Nicowar::AINicowar*)players[3]->ai->aiImplementation)->getGradientManager().getGradient(Nicowar::Gradient::VillageCenter, Nicowar::Gradient::Resource).getHeight(x+viewportX, y+viewportY));
+//				globalContainer->gfx->drawString((x<<5), (y<<5), globalContainer->littleFont, ((Nicowar::AINicowar*)players[3]->ai->aiImplementation)->getGradientManager().getGradient(Nicowar::Gradient::VillageCenter, Nicowar::Gradient::Resource).getHeight(x+viewportX, y+viewportY));
 				//((AICastor *)players[0].ai->aiImplementation)->wheatCareMap
 			}
 				//if (map.getForbidden(x+viewportX, y+viewportY))
@@ -1861,7 +1874,6 @@ void Game::drawMap(int sx, int sy, int sw, int sh, int viewportX, int viewportY,
 					//globalContainer->gfx->drawString((x<<5), (y<<5), globalContainer->littleFont, map.getGradient(1, 5, 0, x+viewportX, y+viewportY));
 					//globalContainer->gfx->drawString((x<<5), (y<<5), globalContainer->littleFont, map.getGradient(0, STONE, 1, x+viewportX, y+viewportY));
 					//globalContainer->gfx->drawString((x<<5), (y<<5), globalContainer->littleFont, map.forbiddenGradient[0][0][(x+viewportX)+(y+viewportY)*map.w]);
-					
 					//globalContainer->gfx->drawString((x<<5), (y<<5)+16, globalContainer->littleFont, ((x+viewportX)&(map.getMaskW())));
 					//globalContainer->gfx->drawString((x<<5)+16, (y<<5)+8, globalContainer->littleFont, ((y+viewportY)&(map.getMaskH())));
 				//}
@@ -1949,8 +1961,11 @@ void Game::drawMap(int sx, int sy, int sw, int sh, int viewportX, int viewportY,
 				}
 
 	}
+}
 
-	// We draw ground buildings:
+inline void Game::drawMapGroundBuildings(int left, int top, int right, int bot, int sw, int sh, int viewportX, int viewportY, int localTeam, Uint32 drawOptions)
+{
+	std::set<Building *, BuildingPosComp> buildingList;
 	for (int y=top-1; y<=bot; y++)
 		for (int x=left-1; x<=right; x++)
 		{
@@ -2134,8 +2149,10 @@ void Game::drawMap(int sx, int sy, int sw, int sh, int viewportX, int viewportY,
 			globalContainer->gfx->drawString(accessX, accessY, globalContainer->littleFont, oss.str());
 		}
 	}
-		
-	// draw forbidden and guard areas
+}
+
+inline void Game::drawMapAreas(int left, int top, int right, int bot, int sw, int sh, int viewportX, int viewportY, int localTeam, Uint32 drawOptions)
+{
 	if ((drawOptions & DRAW_AREA) != 0)
 		for (int y=top; y<bot; y++)
 			for (int x=left; x<right; x++)
@@ -2174,25 +2191,47 @@ void Game::drawMap(int sx, int sy, int sw, int sh, int viewportX, int viewportY,
 				}
 				if (map.isClearAreaLocal(x+viewportX, y+viewportY))
 				{
-					globalContainer->gfx->drawLine(16+(x<<5), (y<<5), 16+(x<<5), 32+(y<<5), 64, 64, 0);
-					globalContainer->gfx->drawLine(32+(x<<5), (y<<5), 32+(x<<5), 32+(y<<5), 64, 64, 0);
+					globalContainer->gfx->drawLine(16+(x<<5), (y<<5), 16+(x<<5), 32+(y<<5), 128, 128, 0);
+//					globalContainer->gfx->drawLine(32+(x<<5), (y<<5), 32+(x<<5), 32+(y<<5), 128, 128, 0);
 					//globalContainer->gfx->drawLine(8+(x<<5), (y<<5), 8+(x<<5), 32+(y<<5), 128, 128, 0);
 					//globalContainer->gfx->drawLine(24+(x<<5), (y<<5), 24+(x<<5), 32+(y<<5), 128, 128, 0);
 					
 					if (!map.isClearAreaLocal(x+viewportX, y+viewportY-1))
-						globalContainer->gfx->drawHorzLine((x<<5), (y<<5), 32, 128, 128, 0);
+						globalContainer->gfx->drawHorzLine((x<<5), (y<<5), 32, 255, 255, 0);
 					if (!map.isClearAreaLocal(x+viewportX, y+viewportY+1))
-						globalContainer->gfx->drawHorzLine((x<<5), 32+(y<<5), 32, 128, 128, 0);
+						globalContainer->gfx->drawHorzLine((x<<5), 32+(y<<5), 32, 255, 255, 0);
 					
 					if (!map.isClearAreaLocal(x+viewportX-1, y+viewportY))
-						globalContainer->gfx->drawVertLine((x<<5), (y<<5), 32, 128, 128, 0);
+						globalContainer->gfx->drawVertLine((x<<5), (y<<5), 32, 255, 255, 0);
 					if (!map.isClearAreaLocal(x+viewportX+1, y+viewportY))
-						globalContainer->gfx->drawVertLine(32+(x<<5), (y<<5), 32, 128, 128, 0);
+						globalContainer->gfx->drawVertLine(32+(x<<5), (y<<5), 32, 255, 255, 0);
+				}
+
+				if((drawOptions & DRAW_NO_RESSOURCE_GROWTH_AREAS) != 0)
+				{
+					if(!map.canRessourcesGrow(x+viewportX, y+viewportY))
+					{
+						globalContainer->gfx->drawLine((x<<5), 8+(y<<5), 32+(x<<5), 8+(y<<5), 128, 64, 0);
+						globalContainer->gfx->drawLine((x<<5), 16+(y<<5), 32+(x<<5), 16+(y<<5), 128, 64, 0);
+						globalContainer->gfx->drawLine((x<<5), 24+(y<<5), 32+(x<<5), 24+(y<<5), 128, 64, 0);
+//						globalContainer->gfx->drawLine((x<<5), 32+(y<<5), 32+(x<<5), 32+(y<<5), 128, 64, 0);
+					
+						if (map.canRessourcesGrow(x+viewportX, y+viewportY-1))
+							globalContainer->gfx->drawHorzLine((x<<5), (y<<5), 32, 255, 128, 0);
+						if (map.canRessourcesGrow(x+viewportX, y+viewportY+1))
+							globalContainer->gfx->drawHorzLine((x<<5), 32+(y<<5), 32, 255, 128, 0);
+						
+						if (map.canRessourcesGrow(x+viewportX-1, y+viewportY))
+							globalContainer->gfx->drawVertLine((x<<5), (y<<5), 32, 255, 128, 0);
+						if (map.canRessourcesGrow(x+viewportX+1, y+viewportY))
+							globalContainer->gfx->drawVertLine(32+(x<<5), (y<<5), 32, 255, 128, 0);
+						}
 				}
 			}
+}
 
-	
-	// We draw air units:
+inline void Game::drawMapAirUnits(int left, int top, int right, int bot, int sw, int sh, int viewportX, int viewportY, int localTeam, Uint32 drawOptions)
+{
 	for (int y=top-1; y<=bot; y++)
 		for (int x=left-1; x<=right; x++)
 		{
@@ -2200,7 +2239,34 @@ void Game::drawMap(int sx, int sy, int sw, int sh, int viewportX, int viewportY,
 			if (gid!=NOGUID)
 				drawUnit(x, y, gid, viewportX, viewportY, (sw>>5), (sh>>5), localTeam, drawOptions);
 		}
+}
 
+inline void Game::drawMapScriptAreas(int left, int top, int right, int bot, int viewportX, int viewportY)
+{
+		for (int y=top; y<bot; y++)
+			for (int x=left; x<right; x++)
+			{
+				std::stringstream str;
+				for(int n=0; n<9; ++n)
+				{
+					if(map.isPointSet(n, x+viewportX, y+viewportY))
+					{
+						str.str("");
+						str<<n+1;
+						globalContainer->gfx->drawString((x<<5)+(n%3)*10, (y<<5)+(n/3)*10, globalContainer->littleFont, str.str());
+	
+						globalContainer->gfx->drawHorzLine((x<<5), (y<<5), 32, 64, 255, 255);
+						globalContainer->gfx->drawHorzLine((x<<5), 32+(y<<5), 32, 64, 255, 255);
+				
+						globalContainer->gfx->drawVertLine((x<<5), (y<<5), 32, 64, 255, 255);
+						globalContainer->gfx->drawVertLine(32+(x<<5), (y<<5), 32, 64, 255, 255);
+					}
+				}
+			}
+}
+
+inline void Game::drawMapBulletsExplosionsDeathAnimations(int left, int top, int right, int bot, int sw, int sh, int viewportX, int viewportY, int localTeam, Uint32 drawOptions)
+{
 	// Let's paint the bullets and explosions
 	// TODO : optimise : test only possible sectors to show bullets.
 
@@ -2271,16 +2337,10 @@ void Game::drawMap(int sx, int sy, int sw, int sh, int viewportX, int viewportY,
 			}
 		}
 	}
-	
-	// draw cloud shadow if we are in high quality
-	if ((globalContainer->settings.optionFlags & GlobalContainer::OPTION_LOW_SPEED_GFX) == 0)
-	{
-		for (int y=waterStartY + (cloudDisplacement % -512); y<sh; y+=512)
-			for (int x=waterStartX + (cloudDisplacement % -512); x<sw; x+=512)
-				globalContainer->gfx->drawSprite(x, y, globalContainer->terrainCloud, 0);
-	}
-	
-	// draw black & shading
+}
+
+inline void Game::drawMapFogOfWar(int left, int top, int right, int bot, int sw, int sh, int viewportX, int viewportY, int localTeam, Uint32 drawOptions)
+{
 	if ((drawOptions & DRAW_WHOLE_MAP) == 0)
 	{
 		// we have decrease on because we do unalign lookup
@@ -2325,6 +2385,119 @@ void Game::drawMap(int sx, int sy, int sw, int sh, int viewportX, int viewportY,
 				}
 			}
 	}
+}
+
+inline void Game::drawMapOverlayMaps(int left, int top, int right, int bot, int sw, int sh, int viewportX, int viewportY, int localTeam, Uint32 drawOptions)
+{
+	TeamStat* latest=teams[localTeam]->stats.getLatestStat();
+	int overlayMax=0;
+	std::vector<int>* overlayMap=NULL;
+	Color overlayColor;
+	if((drawOptions & DRAW_STARVING_OVERLAY))
+	{
+		overlayMax=latest->starvingMax;
+		overlayMap=&latest->starvingMap;
+		overlayColor=Color(192, 0, 0);
+	}
+	if((drawOptions & DRAW_DAMAGED_OVERLAY))
+	{
+		overlayMax=latest->damagedMax;
+		overlayMap=&latest->damagedMap;
+		overlayColor=Color(192, 0, 0);
+	}
+	if((drawOptions & DRAW_DEFENSE_OVERLAY))
+	{
+		overlayMax=latest->defenseMax;
+		overlayMap=&latest->defenseMap;
+		overlayColor=Color(0, 0, 192);
+	}
+
+	if(overlayMap!=NULL && overlayMap->size())
+	{
+		for (int y=top-1; y<=bot; y++)
+		{
+			for (int x=left-1; x<=right; x++)
+			{
+				int rx=(x+viewportX)%map.getW();
+				int ry=(y+viewportY)%map.getH();
+				if((*overlayMap)[latest->getPos(rx, ry)])
+				{
+					if (globalContainer->settings.optionFlags & GlobalContainer::OPTION_LOW_SPEED_GFX)
+					{
+						const int value_c=(*overlayMap)[latest->getPos(rx, ry)];
+						const int alpha_c=40+int(float(160)/float(overlayMax) * float(value_c));
+						globalContainer->gfx->drawFilledRect((x<<5), (y<<5), 32, 32, Color(overlayColor.r, overlayColor.g, overlayColor.b, alpha_c));
+					}
+					else
+					{
+						const int value_c=(*overlayMap)[latest->getPos(rx, ry)];
+						const int value_u=(*overlayMap)[latest->getPos(rx, ry-1)];
+						const int value_d=(*overlayMap)[latest->getPos(rx, ry+1)];
+						const int value_l=(*overlayMap)[latest->getPos(rx-1, ry)];
+						const int value_r=(*overlayMap)[latest->getPos(rx+1, ry)];
+						const int alpha_c=int(float(200)/float(overlayMax) * float(value_c));
+						const int alpha_u=int(float(200)/float(overlayMax) * float(value_u));
+						const int alpha_d=int(float(200)/float(overlayMax) * float(value_d));
+						const int alpha_l=int(float(200)/float(overlayMax) * float(value_l));
+						const int alpha_r=int(float(200)/float(overlayMax) * float(value_r));
+//						const float edge_u_alpha=float(alpha_u+alpha_c)/2;
+//						const float edge_l_alpha=float(alpha_l+alpha_c)/2;
+//						const float edge_r_alpha=float(alpha_r+alpha_c)/2;
+//						const float edge_d_alpha=float(alpha_d+alpha_c)/2;
+						const float edge_u_alpha=float(alpha_u);
+						const float edge_l_alpha=float(alpha_l);
+						const float edge_r_alpha=float(alpha_r);
+						const float edge_d_alpha=float(alpha_d);
+						const float vertical_increment=float(edge_u_alpha - edge_d_alpha)/float(32);
+						const float horizontal_increment=float(edge_l_alpha - edge_r_alpha)/float(32);
+						for(int px=0; px<4; ++px)
+						{
+							for(int py=0; py<4; ++py)
+							{
+	
+								const int pby=(4-py)*8;
+								const int pbx=(4-px)*8;
+								const float vertical_g=float(vertical_increment*pby+edge_d_alpha);
+								const float horizontal_g=float(horizontal_increment*pbx+edge_r_alpha);
+								int alpha=int(horizontal_g + vertical_g)/2;
+								globalContainer->gfx->drawFilledRect((x<<5)+px*8, (y<<5)+py*8, 8, 8, Color(overlayColor.r, overlayColor.g, overlayColor.b, alpha));
+							}
+						}
+					}
+				}
+			}
+		}
+	}
+}
+
+void Game::drawMap(int sx, int sy, int sw, int sh, int viewportX, int viewportY, int localTeam, Uint32 drawOptions)
+{
+	static int time = 0;
+	int left=(sx>>5);
+	int top=(sy>>5);
+	int right=((sx+sw+31)>>5);
+	int bot=((sy+sh+31)>>5);
+	time++;
+
+	drawMapWater(sw, sh, viewportX, viewportY, time);
+	drawMapTerrain(left, top, right, bot, viewportX, viewportY, localTeam, drawOptions);
+	drawMapRessources(left, top, right, bot, viewportX, viewportY, localTeam, drawOptions);
+	drawMapGroundUnits(left, top, right, bot, sw, sh, viewportX, viewportY, localTeam, drawOptions);
+	drawMapDebugAreas(left, top, right, bot, sw, sh, viewportX, viewportY, localTeam, drawOptions);
+	drawMapGroundBuildings(left, top, right, bot, sw, sh, viewportX, viewportY, localTeam, drawOptions);
+	drawMapAreas(left, top, right, bot, sw, sh, viewportX, viewportY, localTeam, drawOptions);
+	drawMapAirUnits(left, top, right, bot, sw, sh, viewportX, viewportY, localTeam, drawOptions);
+	if((drawOptions & DRAW_SCRIPT_AREAS) != 0)
+		drawMapScriptAreas(left, top, right, bot, viewportX, viewportY);
+	drawMapBulletsExplosionsDeathAnimations(left, top, right, bot, sw, sh, viewportX, viewportY, localTeam, drawOptions);
+	// draw cloud shadow if we are in high quality
+	if ((globalContainer->settings.optionFlags & GlobalContainer::OPTION_LOW_SPEED_GFX) == 0)
+	{
+		static DynamicClouds ds(&globalContainer->settings);
+		ds.render(globalContainer->gfx, viewportX, viewportY, sw, sh, time);
+	}
+	drawMapFogOfWar(left, top, right, bot, sw, sh, viewportX, viewportY, localTeam, drawOptions);
+	drawMapOverlayMaps(left, top, right, bot, sw, sh, viewportX, viewportY, localTeam, drawOptions);
 
 	// we look on the whole map for buildings
 	// TODO : increase speed, do not count on graphic clipping
@@ -2419,7 +2592,7 @@ void Game::drawMap(int sx, int sy, int sw, int sh, int viewportX, int viewportY,
 					}
 }
 
-void Game::drawMiniMap(int sx, int sy, int sw, int sh, int viewportX, int viewportY, int localTeam)
+void Game::drawMiniMap(int sx, int sy, int sw, int sh, int viewportX, int viewportY, int localTeam, Uint32 drawOptions)
 {
 	// draw the prerendered minimap, decide if we use low speed graphics or nor
 	if (globalContainer->settings.optionFlags & GlobalContainer::OPTION_LOW_SPEED_GFX)
