@@ -55,6 +55,15 @@ public:
 		DRAW_AREA = 0x8,
 		DRAW_WHOLE_MAP = 0x10,
 		DRAW_ACCESSIBILITY = 0x20,
+		DRAW_SCRIPT_AREAS = 0x40,
+		DRAW_NO_RESSOURCE_GROWTH_AREAS = 0x80,
+		DRAW_STARVING_OVERLAY = 0x100,
+		DRAW_DAMAGED_OVERLAY = 0x200,
+		DRAW_DEFENSE_OVERLAY = 0x400,
+	};
+
+	enum MinimapDrawOption
+	{
 	};
 	
 	struct BuildProject
@@ -63,6 +72,8 @@ public:
 		int posY;
 		int teamNumber;
 		int typeNum;
+		int unitWorking;
+		int unitWorkingFuture;
 	};
 
 public:
@@ -120,10 +131,12 @@ public:
 
 	//void addUnit(int x, int y, int team, int type, int level);
 	Unit *addUnit(int x, int y, int team, int type, int level, int delta, int dx, int dy);
-	Building *addBuilding(int x, int y, int typeNum, int teamNumber);
+	Building *addBuilding(int x, int y, int typeNum, int teamNumber, Sint32 unitWorking = 1, Sint32 unitWorkingFuture = 1);
 	//! This remove anything at case(x, y), and return a rect which include every removed things.
 	bool removeUnitAndBuildingAndFlags(int x, int y, unsigned flags=DEL_UNIT|DEL_BUILDING|DEL_FLAG);
 	bool removeUnitAndBuildingAndFlags(int x, int y, int size, unsigned flags=DEL_UNIT|DEL_BUILDING|DEL_FLAG);
+	///A convenience function, returns a pointer to the unit with the guid, or NULL otherwise
+	Unit* getUnit(int guid);
 
 	bool checkRoomForBuilding(int mousePosX, int mousePosY, const BuildingType *bt, int *buildingPosX, int *buildingPosY, int teamNumber, bool checkFow=true);
 	bool checkRoomForBuilding(int x, int y, const BuildingType *bt, int teamNumber, bool checkFow=true);
@@ -132,7 +145,22 @@ public:
 
 	void drawUnit(int x, int y, Uint16 gid, int viewportX, int viewportY, int screenW, int screenH, int localTeam, Uint32 drawOptions);
 	void drawMap(int sx, int sy, int sw, int sh, int viewportX, int viewportY, int teamSelected, Uint32 drawOptions = 0);
-	void drawMiniMap(int sx, int sy, int sw, int sh, int viewportX, int viewportY, int localTeam);
+private:
+	inline void drawMapWater(int sw, int sh, int viewportX, int viewportY, int time);
+	inline void drawMapTerrain(int left, int top, int right, int bot, int viewportX, int viewportY, int localTeam, Uint32 drawOptions);
+	inline void drawMapRessources(int left, int top, int right, int bot, int viewportX, int viewportY, int localTeam, Uint32 drawOptions);
+	inline void drawMapGroundUnits(int left, int top, int right, int bot, int sw, int sh, int viewportX, int viewportY, int localTeam, Uint32 drawOptions);
+	inline void drawMapDebugAreas(int left, int top, int right, int bot, int sw, int sh, int viewportX, int viewportY, int localTeam, Uint32 drawOptions);
+	inline void drawMapGroundBuildings(int left, int top, int right, int bot, int sw, int sh, int viewportX, int viewportY, int localTeam, Uint32 drawOptions);
+	inline void drawMapAreas(int left, int top, int right, int bot, int sw, int sh, int viewportX, int viewportY, int localTeam, Uint32 drawOptions);
+	inline void drawMapAirUnits(int left, int top, int right, int bot, int sw, int sh, int viewportX, int viewportY, int localTeam, Uint32 drawOptions);
+	inline void drawMapScriptAreas(int left, int top, int right, int bot, int viewportX, int viewportY);
+	inline void drawMapBulletsExplosionsDeathAnimations(int left, int top, int right, int bot, int sw, int sh, int viewportX, int viewportY, int localTeam, Uint32 drawOptions);
+	inline void drawMapFogOfWar(int left, int top, int right, int bot, int sw, int sh, int viewportX, int viewportY, int localTeam, Uint32 drawOptions);
+	inline void drawMapOverlayMaps(int left, int top, int right, int bot, int sw, int sh, int viewportX, int viewportY, int localTeam, Uint32 drawOptions);
+	
+public:
+	void drawMiniMap(int sx, int sy, int sw, int sh, int viewportX, int viewportY, int localTeam, Uint32 drawOptions = 0);
 	void renderMiniMap(int teamSelected, const bool useMapDiscovered=false, int step=0, int stepCount=1);
 	Uint32 checkSum(std::vector<Uint32> *checkSumsVector=NULL, std::vector<Uint32> *checkSumsVectorForBuildings=NULL, std::vector<Uint32> *checkSumsVectorForUnits=NULL);
 	
@@ -146,7 +174,6 @@ public:
 	Map map;
 	DrawableSurface *minimap;
 	Mapscript script;
-	std::string nextMap;
 	std::string campaignText;
 	GameGUI *gui;
 	std::list<BuildProject> buildProjects;
