@@ -27,6 +27,7 @@
 #include "Game.h"
 #include "Order.h"
 #include "Brush.h"
+#include "Campaign.h"
 
 namespace GAGCore
 {
@@ -75,6 +76,7 @@ public:
 	GameGUI();
 	~GameGUI();
 
+
 	void init();
 	void adjustInitialViewport();
 	void adjustLocalTeam();
@@ -117,6 +119,9 @@ public:
 
 	// Stats for engine
 	void setCpuLoad(int s);
+
+	/// Sets this game as a campaign game from the provided campaign and the provided mission
+	void setCampaignGame(Campaign& campaign, const std::string& missionName);
 	
 private:
 	// Helper function for key and menu
@@ -144,6 +149,7 @@ private:
 	void drawTextCenter(int x, int y, const char *caption, int i=-1);
 	void drawValueAlignedRight(int y, int v);
 	void drawCosts(int ressources[BASIC_COUNT], Font *font);
+	void drawCheckButton(int x, int y, const char* caption, bool isSet);
 
 	void iterateSelection(void);
 	void centerViewportOnSelection(void);
@@ -255,7 +261,7 @@ private:
 	//! Tells whether a space was clicked recently, to read in by the script engine
 	bool hasSpaceBeenClicked;
 
-	//! When set, tells the gui not to treat clicking the space key as usual, but instead, it will "swallow" it
+	//! When set, tells the gui not to treat clicking the space key as usual, but instead, it will "swallow" (ignore) it
 	bool swallowSpaceKey;
 
 	//! True if the mouse's button way never relased since selection.
@@ -273,6 +279,10 @@ private:
 	//! Coordinate of viewport when began panning
 	int panViewX, panViewY;
 
+	bool showStarvingMap;
+	bool showDamagedMap;
+	bool showDefenseMap;
+
 	bool showUnitWorkingToBuilding;
 
 	TeamStats *teamStats;
@@ -348,6 +358,21 @@ private:
 	#define SMOOTH_CPU_LOAD_WINDOW_LENGTH 32
 	int smoothedCpuLoad[SMOOTH_CPU_LOAD_WINDOW_LENGTH];
 	unsigned smoothedCpuLoadPos;
+
+	// Stuff for the correct working of the campaign
+	Campaign* campaign;
+	std::string missionName;
+
+	// determine building type for custom settings
+	#define NUMBER_BUILDING_TYPE_NUM_WITH_PREDEFINED_UNIT_COUNT 50
+	int unitCount[NUMBER_BUILDING_TYPE_NUM_WITH_PREDEFINED_UNIT_COUNT];
+	
+	//! Init all predefined unit settings to from predefined settings
+	friend class Game;
+	void initUnitCount(void);
+	int getUnitCount(unsigned typeNum);
+	void setUnitCount(unsigned typeNum, int nbReq);
+
 };
 
 #endif
