@@ -27,7 +27,15 @@ using namespace GAGGUI;
 
 class Glob2Style : public Style
 {
+public:
+	Glob2Style();
+	~Glob2Style();
+
+protected:
 	virtual void drawTextButtonBackground(GAGCore::DrawableSurface *target, int x, int y, int w, int h, unsigned highlight);
+	
+protected:
+	Sprite *sprite;
 };
 
 #endif
