@@ -35,6 +35,7 @@
 #include "Race.h"
 #include "Glob2Screen.h"
 #include "UnitsSkins.h"
+#include "Glob2Style.h"
 
 // version related stuff
 #ifdef HAVE_CONFIG_H
@@ -46,6 +47,7 @@
 #include "YOGConsts.h"
 #include "NetConsts.h"
 
+#include "GraphicContext.h"
 
 GlobalContainer::GlobalContainer(void)
 {
@@ -94,6 +96,7 @@ GlobalContainer::GlobalContainer(void)
 GlobalContainer::~GlobalContainer(void)
 {
 	// unlink GUI style
+	if(!runNoX) delete Style::style;
 	Style::style = &defaultStyle;
 	
 	// release unit skins
@@ -210,6 +213,18 @@ void GlobalContainer::parseArgs(int argc, char *argv[])
 				i++;
 			}
 		}
+		if (strcmp(argv[i], "-textshot")==0)
+		{
+			if(i+1 < argc)
+			{
+				GAGCore::DrawableSurface::translationPicturesDirectory = argv[i+1];
+				i++;
+			}
+			else
+			{
+				std::cerr<<"-textshot requires a directory. Use \"-textshot .\" for the current directory."<<std::endl;
+			}
+		}
 		if (strcmp(argv[i], "-f")==0)
 		{
 			settings.screenFlags |= GraphicContext::FULLSCREEN;
@@ -279,7 +294,6 @@ void GlobalContainer::parseArgs(int argc, char *argv[])
 		{
 			printf("\nGlobulation 2\n");
 			printf("Command line arguments:\n");
-			printf("-t\ttype of gfx renderer: 0 = SDL, 1 = OpenGL\n");
 			printf("-s\tset resolution and depth (for instance : -s640x480 or -s640x480x32)\n");
 			printf("-f/-F\tset/clear full screen\n");
 			printf("-r/-R\tset/clear resizable window\n");
@@ -294,6 +308,7 @@ void GlobalContainer::parseArgs(int argc, char *argv[])
 			printf("-y\tspecify an alternative hostname for YOG server\n");
 			printf("-host <map file name> <YOG username> <YOG password>\t runs only as a YOG game host text-based server\n");
 			printf("-nox <game file name> \t runs the game without using the X server\n");
+			printf("-textshot <directory>\t takes pictures of various translation texts as they are drawn on the screen, requires the convert command\n");
 			printf("-vs <name>\tsave a videoshot as name\n");
 			printf("-version\tprint the version and exit\n");
 			exit(0);
@@ -455,8 +470,8 @@ void GlobalContainer::load(void)
 		updateLoadProgressBar(35);
 		
 		// load fonts
-		Toolkit::loadFont("data/fonts/sans.ttf", 22, "menu");
-		Toolkit::loadFont("data/fonts/sans.ttf", 14, "standard");
+		Toolkit::loadFont("data/fonts/sans.ttf", 20, "menu");
+		Toolkit::loadFont("data/fonts/sans.ttf", 13, "standard");
 		Toolkit::loadFont("data/fonts/sans.ttf", 10, "little");
 		menuFont = Toolkit::getFont("menu");
 		menuFont->setStyle(Font::Style(Font::STYLE_NORMAL, GAGGUI::ColorTheme::frontColor));
@@ -496,7 +511,9 @@ void GlobalContainer::load(void)
 		gamegui = Toolkit::getSprite("data/gfx/gamegui");
 		brush = Toolkit::getSprite("data/gfx/brush");
 		magiceffect = Toolkit::getSprite("data/gfx/magiceffect");
-		Style::style = &style;
+		
+		// use custom style
+		Style::style = new Glob2Style;
 
 		updateLoadProgressBar(100);
 		destroyProgressBar();
