@@ -2,6 +2,8 @@
   Copyright (C) 2001-2004 Stephane Magnenat & Luc-Olivier de Charrière
   for any question or comment contact us at nct@ysagoon.com or nuage@ysagoon.com
 
+  Copyright (C) 2006 Bradley Arsenault
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
@@ -55,8 +57,14 @@ struct Case
 	Uint16 airUnit;
 
 	Uint32 forbidden; // This is a mask, one bit by team, 1=forbidden, 0=allowed
+	///The difference between forbidden zone and hidden forbidden zone is that hidden forbidden zone
+	///is put there by the game engine and is not draw to the screen.
+	Uint32 hiddenForbidden; // This is a mask, one bit by team, 1=forbidden, 0=allowed
 	Uint32 guardArea; // This is a mask, one bit by team, 1=guard area, 0=normal
 	Uint32 clearArea; // This is a mask, one bit by team, 1=clear area, 0=normal
+
+	Uint16 scriptAreas; // This is also a mask. A single bit represents an area #n, on or off for the square
+	Uint8 canRessourcesGrow; // This is a boolean, it represents whether ressources are allowed to grow into this location.
 };
 
 
@@ -118,6 +126,18 @@ public:
 	//! Return the number of sectors on y, which corresponds to the sector map height
 	int getSectorH(void) const { return hSector; }
 
+	///Returns a normalized version of the x cordinate, taking into account that x cordinates wrap arround
+	int normalizeX(int x)
+	{
+		return (x+w)%w;
+	}
+	
+	///Returns a normalized version of the y cordinate, taking into account that y cordinates wrap arround
+	int normalizeY(int y)
+	{
+		return (y+h)%h;
+	}
+
 	//! Set map to discovered state at position (x, y) for all teams in sharedVision (mask).
 	void setMapDiscovered(int x, int y, Uint32 sharedVision)
 	{
@@ -236,7 +256,7 @@ public:
 	}
 	
 	// note - these are only meant to be called for the LOCAL team
-	// (the one whose stuff is displayed on the screen, etc)
+	// (the one whose stuff is displayed on the screen)
 	//! Compute localForbiddenMap from cases array
 	void computeLocalForbidden(int localTeamNo);
 	//! Compute localGuardAreaMap from cases array
@@ -286,9 +306,10 @@ public:
 		return (cases+pos)->ressource;
 	}
 	
+	//Returns the combined forbidden and hidden foribidden masks
 	Uint32 getForbidden(int x, int y)
 	{
-		return cases[((y&hMask)<<wDec)+(x&wMask)].forbidden;
+		return cases[((y&hMask)<<wDec)+(x&wMask)].forbidden | cases[((y&hMask)<<wDec)+(x&wMask)].hiddenForbidden;
 	}
 	
 	Uint8 getExplored(int x, int y, int team)
@@ -311,6 +332,28 @@ public:
 		cases[((y&hMask)<<wDec)+(x&wMask)].forbidden = forbidden;
 	}
 	
+	void addForbidden(int x, int y, Uint32 teamNum)
+	{
+		cases[((y&hMask)<<wDec)+(x&wMask)].forbidden |=  Team::teamNumberToMask(teamNum);
+	}
+
+	void removeForbidden(int x, int y, Uint32 teamNum)
+	{
+		Case& c=cases[((y&hMask)<<wDec)+(x&wMask)];
+		c.hiddenForbidden ^= c.forbidden &  Team::teamNumberToMask(teamNum);
+	}
+	
+	void addHiddenForbidden(int x, int y, Uint32 teamNum)
+	{
+		cases[((y&hMask)<<wDec)+(x&wMask)].hiddenForbidden |= Team::teamNumberToMask(teamNum);
+	}
+
+	void removeHiddenForbidden(int x, int y, Uint32 teamNum)
+	{
+		Case& c=cases[((y&hMask)<<wDec)+(x&wMask)];
+		c.hiddenForbidden ^= c.hiddenForbidden &  Team::teamNumberToMask(teamNum);
+	}
+	
 	bool isWater(int x, int y)
 	{
 		int t = getTerrain(x, y)-256;
@@ -363,6 +406,11 @@ public:
 		return true;
 	}
 
+	bool canRessourcesGrow(int x, int y)
+	{
+		return getCase(x, y).canRessourcesGrow;
+	}
+
 	//! Decrement ressource at position (x,y). Return true on success, false otherwise.
 	void decRessource(int x, int y);
 	//! Decrement ressource at position (x,y) if ressource type = ressourceType. Return true on success, false otherwise.
@@ -430,6 +478,23 @@ public:
 	void setRessource(int x, int y, int type, int l);
 	bool isRessourceAllowed(int x, int y, int type);
 	
+
+	///The following is for script areas, which are named areas for map scripts set in the editor
+	///@{
+	///Returns whether area #n is set for a particular point. n can be from 0 to 8
+	bool isPointSet(int n, int x, int y);
+	///Sets a particular point on area #n
+	void setPoint(int n, int x, int y);
+	///Unsets a particular point on area #n
+	void unsetPoint(int n, int x, int y);
+	///Returns the name of area #n
+	std::string getAreaName(int n);
+	///Sets the name of area #n
+	void setAreaName(int n, std::string name);
+	///A vector holding the area names
+	std::vector<std::string> areaNames;
+	///@}
+	
 	//! Transform coordinate from map scale (mx,my) to pixel scale (px,py)
 	void mapCaseToPixelCase(int mx, int my, int *px, int *py) { *px=(mx<<5); *py=(my<<5); }
 	//! Transform coordinate from map (mx,my) to screen (px,py). Use this one to display a building or an unit to the screen.
@@ -633,16 +698,17 @@ public:
 	
 public:
 	// Used to go to ressources
-	//[int team][int ressourceNumber][bool unitCanSwim][int mapX][int mapY]
-	//255=ressource, 0=obstacle, the higher it is, the closest it is from the ressouce.
+	//TODO: make the 32 into a variable TEAM_COUNT to save memory in almost all cases
+	//[int team][int ressourceNumber][bool unitCanSwim]
+	//255=ressource, 0=obstacle, the higher it is, the closer it is to the ressouce.
 	Uint8 *ressourcesGradient[32][MAX_NB_RESSOURCES][2];
 	
 	// Used to go out of forbidden areas
-	//[int team][bool unitCanSwim][int mapX][int mapY]
+	//[int team][bool unitCanSwim]
 	Uint8 *forbiddenGradient[32][2];
 	
 	// Used to attrack idle warriors into guard areas
-	//[int team][bool unitCanSwim][int mapX][int mapY]
+	//[int team][bool unitCanSwim]
 	Uint8 *guardAreasGradient[32][2];
 	
 	// Used to guide explorers
