@@ -31,8 +31,10 @@ using namespace GAGGUI;
 
 HowNewMapScreen::HowNewMapScreen()
 {
-	addWidget(new TextButton(0,  70, 300, 40, ALIGN_CENTERED, ALIGN_SCREEN_CENTERED, "data/gfx/gamegui", 26, 27, "menu", Toolkit::getStringTable()->getString("[new]"), NEW, 13));
-	addWidget(new TextButton(0,  130, 300, 40, ALIGN_CENTERED, ALIGN_SCREEN_CENTERED, "data/gfx/gamegui", 26, 27, "menu", Toolkit::getStringTable()->getString("[load]"), LOAD));
+	addWidget(new TextButton(0,  70, 300, 40, ALIGN_CENTERED, ALIGN_SCREEN_CENTERED, "data/gfx/gamegui", 26, 27, "menu", Toolkit::getStringTable()->getString("[new]"), NEWMAP, 13));
+	addWidget(new TextButton(0,  130, 300, 40, ALIGN_CENTERED, ALIGN_SCREEN_CENTERED, "data/gfx/gamegui", 26, 27, "menu", Toolkit::getStringTable()->getString("[load]"), LOADMAP));
+	addWidget(new TextButton(0, 190, 300, 40,  ALIGN_CENTERED, ALIGN_SCREEN_CENTERED, "data/gfx/gamegui", 26, 27, "menu", Toolkit::getStringTable()->getString("[new campaign]"), NEWCAMPAIGN));
+	addWidget(new TextButton(0, 250, 300, 40,  ALIGN_CENTERED, ALIGN_SCREEN_CENTERED, "data/gfx/gamegui", 26, 27, "menu", Toolkit::getStringTable()->getString("[load campaign]"), LOADCAMPAIGN));
 	addWidget(new TextButton(0, 415, 300, 40, ALIGN_CENTERED, ALIGN_SCREEN_CENTERED, "data/gfx/gamegui", 26, 27, "menu", Toolkit::getStringTable()->getString("[goto main menu]"), CANCEL, 27));
 	addWidget(new Text(0, 18, ALIGN_FILL, ALIGN_SCREEN_CENTERED, "menu", Toolkit::getStringTable()->getString("[editor]")));
 }
@@ -41,7 +43,7 @@ void HowNewMapScreen::onAction(Widget *source, Action action, int par1, int par2
 {
 	if ((action==BUTTON_RELEASED) || (action==BUTTON_SHORTCUT))
 	{
-		if ((par1==NEW)||(par1==LOAD)||(par1==CANCEL))
+		if ((par1==NEWMAP)||(par1==LOADMAP)||(par1==CANCEL) || (par1==NEWCAMPAIGN) || (par1==LOADCAMPAIGN))
 			endExecute(par1);
 	}
 }
@@ -66,6 +68,16 @@ NewMapScreen::NewMapScreen()
 	mapSizeY->add(512);
 	mapSizeY->setNth(descriptor.hDec-6);
 	addWidget(mapSizeY);
+
+	logRepeatAreaTimes=new Number(310, 75, 114, 20, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, 20, "menu");
+	logRepeatAreaTimes->add(1);
+	logRepeatAreaTimes->add(2);
+	logRepeatAreaTimes->add(4);
+	logRepeatAreaTimes->add(8);
+	logRepeatAreaTimes->add(16);
+	logRepeatAreaTimes->add(32);
+	logRepeatAreaTimes->visible=false;
+	addWidget(logRepeatAreaTimes);
 	
 	methodes=new List(20, 100, 280, 300, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "menu");
 	methodes->addText(Toolkit::getStringTable()->getString("[uniform terrain]"));
@@ -127,6 +139,10 @@ NewMapScreen::NewMapScreen()
 	numberOfTeamText->visible=false;
 	addWidget(numberOfTeamText);
 
+	areaTimesText=new Text(430, 75, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", Toolkit::getStringTable()->getString("[repeat area]"));
+	areaTimesText->visible=false;
+	addWidget(areaTimesText);
+
 	numberOfWorkerText=new Text (430, 120, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", Toolkit::getStringTable()->getString("[workers]"));
 	numberOfWorkerText->visible=false;
 	addWidget(numberOfWorkerText);
@@ -255,7 +271,6 @@ NewMapScreen::NewMapScreen()
 	smoothingText=new Text(480, 360, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", Toolkit::getStringTable()->getString("[smoothing]"));
 	smoothingText->visible=false;
 	addWidget(smoothingText);
-	
 	// eOLDISLANDS
 
 	oldIslandSize=new Ratio(310, 140, 114, 18, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, 40, descriptor.oldIslandSize, "menu");
@@ -311,6 +326,8 @@ void NewMapScreen::onAction(Widget *source, Action action, int par1, int par2)
 		// eOLDISLANDS
 		descriptor.oldBeach=oldBeach->getNth();
 		descriptor.nbWorkers=nbWorkers->getNth()+1;
+
+		descriptor.logRepeatAreaTimes=logRepeatAreaTimes->getNth();
 	}
 	else if (action==LIST_ELEMENT_SELECTED)
 	{
@@ -345,6 +362,7 @@ void NewMapScreen::onAction(Widget *source, Action action, int par1, int par2)
 				oldIslandSize->visible=oldIslandSizeText->visible=
 				riverDiameter->visible=riverDiameterText->visible=
 				fruitRatio->visible=fruitText->visible=
+				logRepeatAreaTimes->visible=areaTimesText->visible=
 				false;
 			
 				// not eUNIFORM
@@ -368,6 +386,7 @@ void NewMapScreen::onAction(Widget *source, Action action, int par1, int par2)
 						stoneRatio->visible=stoneText->visible=
 						algaeRatio->visible=algaeText->visible=
 						fruitRatio->visible=fruitText->visible=
+						logRepeatAreaTimes->visible=areaTimesText->visible=
 						true;
 						break;
 					case  MapGenerationDescriptor::eRIVER:
@@ -383,6 +402,7 @@ void NewMapScreen::onAction(Widget *source, Action action, int par1, int par2)
 						algaeRatio->visible=algaeText->visible=
 						riverDiameter->visible=riverDiameterText->visible=
 						fruitRatio->visible=fruitText->visible=
+						logRepeatAreaTimes->visible=areaTimesText->visible=
 						true;
 						break;
 					case  MapGenerationDescriptor::eISLANDS:
@@ -398,6 +418,7 @@ void NewMapScreen::onAction(Widget *source, Action action, int par1, int par2)
 						algaeRatio->visible=algaeText->visible=
 						extraIslands->visible=extraIslandsText->visible=
 						fruitRatio->visible=fruitText->visible=
+						logRepeatAreaTimes->visible=areaTimesText->visible=
 						true;
 						break;
 					case  MapGenerationDescriptor::eCRATERLAKES:
@@ -413,6 +434,7 @@ void NewMapScreen::onAction(Widget *source, Action action, int par1, int par2)
 						algaeRatio->visible=algaeText->visible=
 						craterDensity->visible=craterDensityText->visible=
 						fruitRatio->visible=fruitText->visible=
+						logRepeatAreaTimes->visible=areaTimesText->visible=
 						true;
 						break;
 					case  MapGenerationDescriptor::eOLDRANDOM:
