@@ -37,9 +37,11 @@ class HowNewMapScreen : public Glob2Screen
 public:
 	enum
 	{
-		NEW = 1,
-		LOAD = 2,
-		CANCEL = 3
+		NEWMAP = 1,
+		LOADMAP = 2,
+		CANCEL = 3,
+		NEWCAMPAIGN = 4,
+		LOADCAMPAIGN = 5,
 	};
 
 public:
@@ -74,8 +76,9 @@ private:
 	Ratio *oldIslandSize;
 	Number *oldBeach;
 	Number *nbWorkers;
+	Number *logRepeatAreaTimes;
 	Text *numberOfTeamText, *numberOfWorkerText, *craterDensityText, *extraIslandsText;
-	Text *ratioText, *waterText, *sandText, *grassText, *desertText, *wheatText, *woodText, *stoneText, *algaeText, *fruitText, *smoothingText, *riverDiameterText;
+	Text *ratioText, *waterText, *sandText, *grassText, *desertText, *wheatText, *woodText, *stoneText, *algaeText, *fruitText, *smoothingText, *riverDiameterText, *areaTimesText;
 	Text *oldIslandSizeText, *oldBeachSizeText;
 	
 
