@@ -171,17 +171,19 @@ Order *Order::getOrder(const Uint8 *netData, int netDataLength)
 OrderCreate::OrderCreate(const Uint8 *data, int dataLength)
 :Order()
 {
-	assert(dataLength==16);
+	assert(dataLength==24);//if changed don't forget order.h update
 	bool good=setData(data, dataLength);
 	assert(good);
 }
 
-OrderCreate::OrderCreate(Sint32 teamNumber, Sint32 posX, Sint32 posY, Sint32 typeNum)
+OrderCreate::OrderCreate(Sint32 teamNumber, Sint32 posX, Sint32 posY, Sint32 typeNum, Sint32 unitWorking, Sint32 unitWorkingFuture)
 {
 	this->teamNumber=teamNumber;
 	this->posX=posX;
 	this->posY=posY;
 	this->typeNum=typeNum;
+	this->unitWorking=unitWorking;
+	this->unitWorkingFuture=unitWorkingFuture;
 }
 
 Uint8 *OrderCreate::getData(void)
@@ -192,6 +194,8 @@ Uint8 *OrderCreate::getData(void)
 	addSint32(data, this->posX, 4);
 	addSint32(data, this->posY, 8);
 	addSint32(data, this->typeNum, 12);
+	addSint32(data, this->unitWorking, 16);
+	addSint32(data, this->unitWorkingFuture, 20);
 	
 	return data;
 }
@@ -205,6 +209,8 @@ bool OrderCreate::setData(const Uint8 *data, int dataLength)
 	this->posX=getSint32(data, 4);
 	this->posY=getSint32(data, 8);
 	this->typeNum=getSint32(data, 12);
+	this->unitWorking=getSint32(data, 16);
+	this->unitWorkingFuture=getSint32(data, 20);
 	
 	memcpy(this->data, data, dataLength);
 	
@@ -279,21 +285,25 @@ bool OrderCancelDelete::setData(const Uint8 *data, int dataLength)
 OrderConstruction::OrderConstruction(const Uint8 *data, int dataLength)
 :Order()
 {
-	assert(dataLength==2);
+	assert(dataLength==10);
 	bool good=setData(data, dataLength);
 	assert(good);
 }
 
-OrderConstruction::OrderConstruction(Uint16 gid)
+OrderConstruction::OrderConstruction(Uint16 gid, Uint32 unitWorking, Uint32 unitWorkingFuture)
 {
 	assert(gid<32768);
 	this->gid=gid;
+	this->unitWorking=unitWorking;
+	this->unitWorkingFuture=unitWorkingFuture;
 }
 
 Uint8 *OrderConstruction::getData(void)
 {
 	assert(sizeof(data) == getDataLength());
 	addUint16(data, this->gid, 0);
+	addUint32(data, this->unitWorking, 2);
+	addUint32(data, this->unitWorkingFuture, 6);
 	return data;
 }
 
@@ -302,6 +312,8 @@ bool OrderConstruction::setData(const Uint8 *data, int dataLength)
 	if (dataLength!=getDataLength())
 		return false;
 	this->gid=getUint16(data, 0);
+	this->unitWorking=getUint32(data, 2);
+	this->unitWorkingFuture=getUint32(data, 6);
 	memcpy(this->data, data, dataLength);
 	return true;
 }
