@@ -62,20 +62,22 @@ class OrderCreate:public Order
 {
 public:
 	OrderCreate(const Uint8 *data, int dataLength);
-	OrderCreate(Sint32 teamNumber, Sint32 posX, Sint32 posY, Sint32 typeNum);
+	OrderCreate(Sint32 teamNumber, Sint32 posX, Sint32 posY, Sint32 typeNum, Sint32 unitWorking, Sint32 unitWorkingFuture);
 	virtual ~OrderCreate(void) {}
 	Uint8 getOrderType(void) { return ORDER_CREATE; }
 	Uint8 *getData(void);
 	bool setData(const Uint8 *data, int dataLength);
-	int getDataLength(void) { return 16; }
+	int getDataLength(void) { return 24; }
 
 	Sint32 teamNumber;
 	Sint32 posX;
 	Sint32 posY;
 	Sint32 typeNum;
+	Sint32 unitWorking;
+	Sint32 unitWorkingFuture;
 
  private:
-	Uint8 data[16];
+	Uint8 data[24];
 };
 
 
@@ -115,22 +117,24 @@ protected:
 	Uint8 data[2];
 };
 
-//! Upgarde or Repair a building
+// Upgrade or Repair a building
 class OrderConstruction:public Order
 {
 public:
 	OrderConstruction(const Uint8 *data, int dataLength);
-	OrderConstruction(Uint16 gid);
+	OrderConstruction(Uint16 gid, Uint32 unitWorking, Uint32 unitWorkingFuture);
 	virtual ~OrderConstruction(void) {}
 	Uint8 getOrderType(void) { return ORDER_CONSTRUCTION; }
 	Uint8 *getData(void);
 	bool setData(const Uint8 *data, int dataLength);
-	int getDataLength(void) { return 2; }
+	int getDataLength(void) { return 10; }
 
 	Uint16 gid;
+	Uint32 unitWorking;
+	Uint32 unitWorkingFuture;
 
 protected:
-	Uint8 data[2];
+	Uint8 data[10];
 };
 
 //! Cancel a building upgarde or repair if pending
