@@ -25,6 +25,7 @@
 #include <GAG.h>
 #include <map>
 #include <fstream>
+#include "boost/lexical_cast.hpp"
 
 using namespace GAGCore;
 
@@ -49,11 +50,61 @@ Settings::Settings()
 	defaultLanguage = 0;
 	musicVolume = 255;
 	mute = 0;
-	rememberUnit = 0;
-	warflagUnit = 1;
-	clearflagUnit = 1;
-	exploreflagUnit = 1;
+	rememberUnit = 1;
 	restoreDefaultShortcuts();
+	tempUnit = 1;
+	tempUnitFuture = 1;
+	//whie a little ugly this is to prevent session.cpp from overriding varPrestige once map is chosen
+	tempVarPrestige = 0;
+	for(int n=0; n<IntBuildingType::NB_BUILDING; ++n)
+	{
+		for(int t=0; t<6; ++t)
+		{
+			defaultUnitsAssigned[n][t] = 0;
+		}
+	}
+	defaultUnitsAssigned[IntBuildingType::WAR_FLAG][0] = 20;
+	defaultUnitsAssigned[IntBuildingType::CLEARING_FLAG][0] = 2;
+	defaultUnitsAssigned[IntBuildingType::EXPLORATION_FLAG][0] = 1;
+	defaultUnitsAssigned[IntBuildingType::SWARM_BUILDING][0] = 7;
+	defaultUnitsAssigned[IntBuildingType::SWARM_BUILDING][1] = 4;
+	defaultUnitsAssigned[IntBuildingType::FOOD_BUILDING][0] = 3;
+	defaultUnitsAssigned[IntBuildingType::FOOD_BUILDING][1] = 2;
+	defaultUnitsAssigned[IntBuildingType::FOOD_BUILDING][2] = 5;
+	defaultUnitsAssigned[IntBuildingType::FOOD_BUILDING][3] = 3;
+	defaultUnitsAssigned[IntBuildingType::FOOD_BUILDING][4] = 15;
+	defaultUnitsAssigned[IntBuildingType::FOOD_BUILDING][5] = 8;
+	defaultUnitsAssigned[IntBuildingType::HEAL_BUILDING][0] = 2;
+	defaultUnitsAssigned[IntBuildingType::HEAL_BUILDING][2] = 4;
+	defaultUnitsAssigned[IntBuildingType::HEAL_BUILDING][4] = 6;
+	defaultUnitsAssigned[IntBuildingType::WALKSPEED_BUILDING][0] = 3;
+	defaultUnitsAssigned[IntBuildingType::WALKSPEED_BUILDING][2] = 7;
+	defaultUnitsAssigned[IntBuildingType::WALKSPEED_BUILDING][4] = 12;
+	defaultUnitsAssigned[IntBuildingType::SWIMSPEED_BUILDING][0] = 2;
+	defaultUnitsAssigned[IntBuildingType::SWIMSPEED_BUILDING][2] = 5;
+	defaultUnitsAssigned[IntBuildingType::SWIMSPEED_BUILDING][4] = 12;
+	defaultUnitsAssigned[IntBuildingType::ATTACK_BUILDING][0] = 3;
+	defaultUnitsAssigned[IntBuildingType::ATTACK_BUILDING][2] = 6;
+	defaultUnitsAssigned[IntBuildingType::ATTACK_BUILDING][4] = 9;
+	defaultUnitsAssigned[IntBuildingType::SCIENCE_BUILDING][0] = 5;
+	defaultUnitsAssigned[IntBuildingType::SCIENCE_BUILDING][2] = 10;
+	defaultUnitsAssigned[IntBuildingType::SCIENCE_BUILDING][4] = 20;
+	defaultUnitsAssigned[IntBuildingType::DEFENSE_BUILDING][0] = 3;
+	defaultUnitsAssigned[IntBuildingType::DEFENSE_BUILDING][1] = 2;
+	defaultUnitsAssigned[IntBuildingType::DEFENSE_BUILDING][2] = 5;
+	defaultUnitsAssigned[IntBuildingType::DEFENSE_BUILDING][3] = 2;
+	defaultUnitsAssigned[IntBuildingType::DEFENSE_BUILDING][4] = 8;
+	defaultUnitsAssigned[IntBuildingType::DEFENSE_BUILDING][5] = 2;
+	defaultUnitsAssigned[IntBuildingType::STONE_WALL][0] = 1;
+	defaultUnitsAssigned[IntBuildingType::MARKET_BUILDING][0] = 3;
+
+	cloudPatchSize=16;//the bigger the faster the uglier
+	cloudMaxAlpha=130;//the higher the nicer the clouds the harder the units are visible
+	cloudMaxSpeed=3;
+	cloudWindStability=3550;//how much will the wind change
+	cloudStability=1300;//how much will the clouds change shape
+	cloudSize=300;//the bigger the better they look with big Patches. The smaller the better they look with smaller patches
+	cloudHeight=150;//(cloud - ground) / (eyes - ground) * 100 (to get an int value)
 }
 
 
@@ -166,9 +217,24 @@ void Settings::load(const char *filename)
 		READ_PARSED_INT(musicVolume);		
 		READ_PARSED_INT(mute);
 		READ_PARSED_INT(rememberUnit);
-		READ_PARSED_INT(warflagUnit);
-		READ_PARSED_INT(clearflagUnit);
-		READ_PARSED_INT(exploreflagUnit);
+
+		for(int n=0; n<IntBuildingType::NB_BUILDING; ++n)
+		{
+			for(int t=0; t<6; ++t)
+			{
+				std::string keyname="defaultUnitsAssigned["+boost::lexical_cast<std::string>(n)+"]["+boost::lexical_cast<std::string>(t)+"]";
+				if(parsed.find(keyname)!=parsed.end())
+					defaultUnitsAssigned[n][t] = boost::lexical_cast<int>(parsed[keyname]);
+			}
+		}
+
+		READ_PARSED_INT(cloudPatchSize);
+		READ_PARSED_INT(cloudMaxAlpha);
+		READ_PARSED_INT(cloudMaxSpeed);
+		READ_PARSED_INT(cloudWindStability);
+		READ_PARSED_INT(cloudStability);
+		READ_PARSED_INT(cloudSize);
+		READ_PARSED_INT(cloudHeight);
 
 		for(std::map<std::string, std::string>::iterator i=keyboard_shortcuts.begin(); i!=keyboard_shortcuts.end(); ++i)
 		{
@@ -195,27 +261,6 @@ void Settings::save(const char *filename)
 	}
 	else
 	{
-/*
-		f<<"username="<<username<<std::endl;
-		f<<"password="<<password<<std::endl;
-		f<<"screenWidth="<<screenWidth<<std::endl;
-		f<<"screenHeight="<<screenHeight<<std::endl;
-		f<<"screenFlags="<<screenFlags<<std::endl;
-		f<<"optionFlags="<<optionFlags<<std::endl;
-		f<<"defaultLanguage="<<defaultLanguage<<std::endl;
-		f<<"musicVolume="<<musicVolume<<std::endl;
-		f<<"mute="<<mute<<std::endl;
-		f<<"rememberUnit="<<rememberUnit<<std::endl;
-		f<<"warflagUnit="<<warflagUnit<<std::endl;
-		f<<"clearflagUnit="<<clearflagUnit<<std::endl;
-		f<<"exploreflagUnit="<<exploreflagUnit<<std::endl;
-
-		for(std::map<std::string, std::string>::iterator i=keyboard_shortcuts.begin(); i!=keyboard_shortcuts.end(); ++i)
-		{
-			f<<i->first<<"="<<i->second<<std::endl;
-		}
-*/
-
 		Utilities::streamprintf(stream, "username=%s\n", username.c_str());
 		Utilities::streamprintf(stream, "password=%s\n", password.c_str());
 		Utilities::streamprintf(stream, "screenWidth=%d\n", screenWidth);
@@ -226,9 +271,24 @@ void Settings::save(const char *filename)
 		Utilities::streamprintf(stream, "musicVolume=%d\n", musicVolume);
 		Utilities::streamprintf(stream, "mute=%d\n", mute);
 		Utilities::streamprintf(stream, "rememberUnit=%d\n", rememberUnit);
-		Utilities::streamprintf(stream, "warflagUnit=%d\n", warflagUnit);
-		Utilities::streamprintf(stream, "clearflagUnit=%d\n", clearflagUnit);
-		Utilities::streamprintf(stream, "exploreflagUnit=%d\n", exploreflagUnit);
+
+		for(int n=0; n<IntBuildingType::NB_BUILDING; ++n)
+		{
+			for(int t=0; t<6; ++t)
+			{
+				std::string keyname="defaultUnitsAssigned["+boost::lexical_cast<std::string>(n)+"]["+boost::lexical_cast<std::string>(t)+"]";
+				Utilities::streamprintf(stream, "%s=%i\n", keyname.c_str(), defaultUnitsAssigned[n][t]);
+			}
+		}
+
+		Utilities::streamprintf(stream, "cloudPatchSize=%d\n",	cloudPatchSize);
+		Utilities::streamprintf(stream, "cloudMaxAlpha=%d\n",	cloudMaxAlpha);
+		Utilities::streamprintf(stream, "cloudMaxSpeed=%d\n",	cloudMaxSpeed);
+		Utilities::streamprintf(stream, "cloudWindStability=%d\n",	cloudWindStability);
+		Utilities::streamprintf(stream, "cloudStability=%d\n",	cloudStability);
+		Utilities::streamprintf(stream, "cloudSize=%d\n",	cloudSize);
+		Utilities::streamprintf(stream, "cloudHeight=%d\n",	cloudHeight);
+		
 		for(std::map<std::string, std::string>::iterator i=keyboard_shortcuts.begin(); i!=keyboard_shortcuts.end(); ++i)
 		{
 			Utilities::streamprintf(stream, "game_%s=%s\n", i->first.c_str(), i->second.c_str());
