@@ -33,6 +33,7 @@
 #include "SoundMixer.h"
 #include <ostream>
 #include <algorithm>
+#include "boost/lexical_cast.hpp"
 
 SettingsScreen::SettingsScreen()
 {
@@ -49,7 +50,7 @@ SettingsScreen::SettingsScreen()
 	addWidget(keyboardsettings);
 
 	// Screen entry/quit part
-	ok=new TextButton( 440, 360, 180, 40, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "", -1, -1, "menu", Toolkit::getStringTable()->getString("[ok]"), OK, 13);
+	ok=new TextButton( 230, 420, 180, 40, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "", -1, -1, "menu", Toolkit::getStringTable()->getString("[ok]"), OK, 13);
 	addWidget(ok);
 	cancel=new TextButton(440, 420, 180, 40, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, NULL, -1, -1, "menu", Toolkit::getStringTable()->getString("[Cancel]"), CANCEL, 27);
 	addWidget(cancel);
@@ -101,7 +102,6 @@ SettingsScreen::SettingsScreen()
 	addWidget(customcur);
 	customcurText=new Text(260, 90 + 90, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", Toolkit::getStringTable()->getString("[customcur]"), 180);
 	addWidget(customcurText);
-	//TODO currently rememberUnitButton does nothing but is meant to eventually allow the game to remember the last number of globs assigned to a building/flag so the game will not always use the defaults
 	rememberUnitButton=new OnOffButton(230, 90 + 120, 20, 20, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, globalContainer->settings.rememberUnit, REMEMBERUNIT);
 	addWidget(rememberUnitButton);
 	rememberUnitText=new Text(260, 90 + 120, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", Toolkit::getStringTable()->getString("[remember unit]"), 180);
@@ -129,105 +129,56 @@ SettingsScreen::SettingsScreen()
 	addWidget(audioMute);
 	audioMuteText=new Text(260, 360, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", Toolkit::getStringTable()->getString("[mute]"), 200);
 	addWidget(audioMuteText);
-	musicVol=new Selector(230, 420, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, 180, globalContainer->settings.musicVolume, 256);
+	musicVol=new Selector(320, 360, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, 180, globalContainer->settings.musicVolume, 256);
 	addWidget(musicVol);
-	musicVolText=new Text(230, 390, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", Toolkit::getStringTable()->getString("[Music volume]"), 300);
+	musicVolText=new Text(320, 330, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", Toolkit::getStringTable()->getString("[Music volume]"), 300);
 	addWidget(musicVolText);
 	setVisibilityFromAudioSettings();
 	
-	//following are all unit settings
-	warflagUnitRatio=new Number(20, 80, 100, 18, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, 20, "menu");
-	warflagUnitRatio->add(0);
-	warflagUnitRatio->add(1);
-	warflagUnitRatio->add(2);
-	warflagUnitRatio->add(3);
-	warflagUnitRatio->add(4);
-	warflagUnitRatio->add(5);
-	warflagUnitRatio->add(6);
-	warflagUnitRatio->add(7);
-	warflagUnitRatio->add(8);
-	warflagUnitRatio->add(9);
-	warflagUnitRatio->add(10);
-	warflagUnitRatio->add(11);
-	warflagUnitRatio->add(12);
-	warflagUnitRatio->add(13);
-	warflagUnitRatio->add(14);
-	warflagUnitRatio->add(15);
-	warflagUnitRatio->add(16);
-	warflagUnitRatio->add(17);
-	warflagUnitRatio->add(18);
-	warflagUnitRatio->add(19);
-	warflagUnitRatio->add(20);
-	warflagUnitRatio->setNth(globalContainer->settings.warflagUnit);
-	warflagUnitRatio->visible=false;
-	addWidget(warflagUnitRatio);
-	
-	warflagUnitText=new Text(20, 60, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", Toolkit::getStringTable()->getString("[war flag]"));
-	addWidget(warflagUnitText);
-	warflagUnitText->visible=false;
+	int row=0;
+	int current_column_x=20;
+	int widest_element=0;
+	for(int t=0; t<IntBuildingType::NB_BUILDING; ++t)
+	{
+		std::string name=IntBuildingType::typeFromShortNumber(t);
+		for(int l=0; l<6; ++l)
+		{
+			if(globalContainer->buildingsTypes.getByType(name, l/2, (l+1)%2) != NULL && globalContainer->settings.defaultUnitsAssigned[t][l]>0)
+			{
+				unitRatios[t][l] = new Number(current_column_x, 70+45*row, 100, 18, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, 20, "menu");
+				addNumbersFor(0, 20, unitRatios[t][l]);
+				unitRatios[t][l]->setNth(globalContainer->settings.defaultUnitsAssigned[t][l]);
+				unitRatios[t][l]->visible=false;
+				addWidget(unitRatios[t][l]);
 
+				widest_element = std::max(widest_element, unitRatios[t][l]->getWidth());
 
-	clearflagUnitRatio=new Number(20, 140, 100, 18, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, 20, "menu");
-	clearflagUnitRatio->add(0);
-	clearflagUnitRatio->add(1);
-	clearflagUnitRatio->add(2);
-	clearflagUnitRatio->add(3);
-	clearflagUnitRatio->add(4);
-	clearflagUnitRatio->add(5);
-	clearflagUnitRatio->add(6);
-	clearflagUnitRatio->add(7);
-	clearflagUnitRatio->add(8);
-	clearflagUnitRatio->add(9);
-	clearflagUnitRatio->add(10);
-	clearflagUnitRatio->add(11);
-	clearflagUnitRatio->add(12);
-	clearflagUnitRatio->add(13);
-	clearflagUnitRatio->add(14);
-	clearflagUnitRatio->add(15);
-	clearflagUnitRatio->add(16);
-	clearflagUnitRatio->add(17);
-	clearflagUnitRatio->add(18);
-	clearflagUnitRatio->add(19);
-	clearflagUnitRatio->add(20);
-	clearflagUnitRatio->setNth(globalContainer->settings.clearflagUnit);
-	clearflagUnitRatio->visible=false;
-	addWidget(clearflagUnitRatio);
-	
-	clearflagUnitText=new Text(20, 120, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", Toolkit::getStringTable()->getString("[clearing flag]"));
-	addWidget(clearflagUnitText);
-	clearflagUnitText->visible=false;
+				std::string keyname="[";
+				if((l+1)%2)
+					keyname+="build ";
+				keyname+=name + " level " + boost::lexical_cast<std::string>(l/2) + "]";
+				unitRatioTexts[t][l]=new Text(current_column_x, 50+45*row, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", Toolkit::getStringTable()->getString(keyname.c_str()));
+				addWidget(unitRatioTexts[t][l]);
+				unitRatioTexts[t][l]->visible=false;
 
-	
-	exploreflagUnitRatio=new Number(20, 200, 100, 18, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, 20, "menu");
-	exploreflagUnitRatio->add(0);
-	exploreflagUnitRatio->add(1);
-	exploreflagUnitRatio->add(2);
-	exploreflagUnitRatio->add(3);
-	exploreflagUnitRatio->add(4);
-	exploreflagUnitRatio->add(5);
-	exploreflagUnitRatio->add(6);
-	exploreflagUnitRatio->add(7);
-	exploreflagUnitRatio->add(8);
-	exploreflagUnitRatio->add(9);
-	exploreflagUnitRatio->add(10);
-	exploreflagUnitRatio->add(11);
-	exploreflagUnitRatio->add(12);
-	exploreflagUnitRatio->add(13);
-	exploreflagUnitRatio->add(14);
-	exploreflagUnitRatio->add(15);
-	exploreflagUnitRatio->add(16);
-	exploreflagUnitRatio->add(17);
-	exploreflagUnitRatio->add(18);
-	exploreflagUnitRatio->add(19);
-	exploreflagUnitRatio->add(20);
-	exploreflagUnitRatio->setNth(globalContainer->settings.exploreflagUnit);
-	exploreflagUnitRatio->visible=false;
-	addWidget(exploreflagUnitRatio);
-	
-	exploreflagUnitText=new Text(20, 180, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard", Toolkit::getStringTable()->getString("[exploration flag]"));
-	addWidget(exploreflagUnitText);
-	exploreflagUnitText->visible=false;
+				widest_element = std::max(widest_element, unitRatioTexts[t][l]->getWidth());
 
+				row+=1;
+				if(row==8)
+				{
+					row=0;
+					current_column_x+= widest_element+10;
+				}
+			}
+			else
+			{
+				unitRatios[t][l]=NULL;
+				unitRatioTexts[t][l]=NULL;
+			}
+		}
+	}
+
+	//shortcuts part
 	game_shortcuts=new TextButton( 20, 60, 200, 40, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "", -1, -1, "menu", Toolkit::getStringTable()->getString("[game shortcuts]"), GAMESHORTCUTS);
 	game_shortcuts->visible=false;
 
@@ -243,7 +194,7 @@ SettingsScreen::SettingsScreen()
 	editor_keyboard_shortcuts=new List(355, 110, 275, 200, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "standard");
 	editor_keyboard_shortcuts->visible=false;
 
-	restore_default_shortcuts = new TextButton(20, 320, 610, 40, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "", -1, -1, "standard", Toolkit::getStringTable()->getString("[restore default shortcuts]"), RESTOREDEFAULTSHORTCUTS);
+	restore_default_shortcuts = new TextButton(20, 315, 610, 40, ALIGN_SCREEN_CENTERED, ALIGN_SCREEN_CENTERED, "", -1, -1, "standard", Toolkit::getStringTable()->getString("[restore default shortcuts]"), RESTOREDEFAULTSHORTCUTS);
 	restore_default_shortcuts->visible=false;
 
 
@@ -335,6 +286,16 @@ SettingsScreen::SettingsScreen()
 	gfxAltered = false;
 }
 
+
+void SettingsScreen::addNumbersFor(int low, int high, Number* widget)
+{
+	for(int i=low; i<=high; ++i)
+	{
+		widget->add(i);
+	}
+}
+
+
 void SettingsScreen::onAction(Widget *source, Action action, int par1, int par2)
 {
 	if ((action==BUTTON_RELEASED) || (action==BUTTON_SHORTCUT))
@@ -381,13 +342,18 @@ void SettingsScreen::onAction(Widget *source, Action action, int par1, int par2)
 			musicVolText->visible=true;
 			rememberUnitButton->visible=true;
 			rememberUnitText->visible=true;
-
-			warflagUnitRatio->visible=false;
-			warflagUnitText->visible=false;
-			clearflagUnitRatio->visible=false;
-			clearflagUnitText->visible=false;
-			exploreflagUnitRatio->visible=false;
-			exploreflagUnitText->visible=false;
+			
+			for(int t=0; t<IntBuildingType::NB_BUILDING; ++t)
+			{
+				for(int l=0; l<6; ++l)
+				{
+					if(unitRatios[t][l])
+					{
+						unitRatios[t][l]->visible=false;
+						unitRatioTexts[t][l]->visible=false;
+					}
+				}
+			}
 
 			game_shortcuts->visible=false;
 			editor_shortcuts->visible=false;
@@ -423,13 +389,18 @@ void SettingsScreen::onAction(Widget *source, Action action, int par1, int par2)
 			rememberUnitButton->visible=false;
 			rememberUnitText->visible=false;
 
-			warflagUnitRatio->visible=true;
-			warflagUnitText->visible=true;
-			clearflagUnitRatio->visible=true;
-			clearflagUnitText->visible=true;
-			exploreflagUnitRatio->visible=true;
-			exploreflagUnitText->visible=true;
-
+			for(int t=0; t<IntBuildingType::NB_BUILDING; ++t)
+			{
+				for(int l=0; l<6; ++l)
+				{
+					if(unitRatios[t][l])
+					{
+						unitRatios[t][l]->visible=true;
+						unitRatioTexts[t][l]->visible=true;
+					}
+				}
+			}
+	
 			game_shortcuts->visible=false;
 			editor_shortcuts->visible=false;
 			keyboard_shortcut_names->visible=false;
@@ -463,13 +434,18 @@ void SettingsScreen::onAction(Widget *source, Action action, int par1, int par2)
 			rememberUnitButton->visible=false;
 			rememberUnitText->visible=false;
 			
-			warflagUnitRatio->visible=false;
-			warflagUnitText->visible=false;
-			clearflagUnitRatio->visible=false;
-			clearflagUnitText->visible=false;
-			exploreflagUnitRatio->visible=false;
-			exploreflagUnitText->visible=false;
-
+			for(int t=0; t<IntBuildingType::NB_BUILDING; ++t)
+			{
+				for(int l=0; l<6; ++l)
+				{
+					if(unitRatios[t][l])
+					{
+						unitRatios[t][l]->visible=false;
+						unitRatioTexts[t][l]->visible=false;
+					}
+				}
+			}
+	
 			game_shortcuts->visible=true;
 			editor_shortcuts->visible=true;
 			keyboard_shortcut_names->visible=true;
@@ -498,21 +474,18 @@ void SettingsScreen::onAction(Widget *source, Action action, int par1, int par2)
 	}
 	else if (action==NUMBER_ELEMENT_SELECTED)
 	{
-		if (warflagUnitRatio->getNth() == 0)
+		for(int t=0; t<IntBuildingType::NB_BUILDING; ++t)
 		{
-			warflagUnitRatio->setNth(1);
+			for(int l=0; l<6; ++l)
+			{
+				if(unitRatios[t][l])
+				{
+					if(unitRatios[t][l]->getNth() == 0)
+						unitRatios[t][l]->setNth(1);
+					globalContainer->settings.defaultUnitsAssigned[t][l]=unitRatios[t][l]->getNth();
+				}
+			}
 		}
-		globalContainer->settings.warflagUnit=warflagUnitRatio->getNth();
-		if (clearflagUnitRatio->getNth() == 0)
-		{
-			clearflagUnitRatio->setNth(1);
-		}
-		globalContainer->settings.clearflagUnit=clearflagUnitRatio->getNth();
-		if (exploreflagUnitRatio->getNth() == 0)
-		{
-			exploreflagUnitRatio->setNth(1);
-		}
-		globalContainer->settings.exploreflagUnit=exploreflagUnitRatio->getNth();
 	}
 	else if (action==LIST_ELEMENT_SELECTED)
 	{
