@@ -65,13 +65,16 @@ private:
 	OnOffButton *fullscreen, *usegpu, *lowquality, *customcur;
 	Selector *musicVol;
 	OnOffButton *audioMute, *rememberUnitButton;
-	Number *warflagUnitRatio, *clearflagUnitRatio, *exploreflagUnitRatio;
+	Number* unitRatios[IntBuildingType::NB_BUILDING][6];
+	Text* unitRatioTexts[IntBuildingType::NB_BUILDING][6];
 //	Text *title;
 	Text *language, *display, *usernameText, *audio;
-	Text *fullscreenText, *usegpuText, *lowqualityText, *customcurText, *musicVolText, *audioMuteText, *warflagUnitText, *clearflagUnitText, *exploreflagUnitText, *rememberUnitText;
+	Text *fullscreenText, *usegpuText, *lowqualityText, *customcurText, *musicVolText, *audioMuteText, *rememberUnitText;
 	Text *actDisplay;
 	Text *rebootWarning;
 
+	void addNumbersFor(int low, int high, Number* widget);
+
 	TextButton* game_shortcuts;
 	TextButton* editor_shortcuts;
 	List* keyboard_shortcut_names;
