@@ -121,10 +121,9 @@ Unit::Unit(int x, int y, Uint16 gid, Sint32 typeNum, Team *team, int level)
 	attachedBuilding=NULL;
 	targetBuilding=NULL;
 	ownExchangeBuilding=NULL;
-	foreingExchangeBuilding=NULL;
 	destinationPurprose=-1;
-	subscribed=false;
 	caryedRessource=-1;
+	jobTimer = 0;
 	
 	// gui
 	levelUpAnimation = 0;
@@ -233,14 +232,19 @@ void Unit::load(GAGCore::InputStream *stream, Team *owner, Sint32 versionMinor)
 	if (versionMinor < 41 && destinationPurprose >= 10)
 		destinationPurprose += 5;
 	
-	subscribed = (bool)stream->readUint32("subscribed");
+	if(versionMinor<56)
+		stream->readUint32("subscribed");
 
 	caryedRessource = stream->readSint32("caryedRessource");
 	
+	if(versionMinor>=56)
+		jobTimer = stream->readSint32("jobTimer");
+
 	// gui
 	levelUpAnimation = 0;
 	magicActionAnimation = 0;
-	
+	jobTimer = 0;
+
 	verbose = false;
 	
 	stream->readLeaveSection();
@@ -308,13 +312,14 @@ void Unit::save(GAGCore::OutputStream *stream)
 	stream->writeSint32(experienceLevel, "experienceLevel");
 
 	stream->writeSint32(destinationPurprose, "destinationPurprose");
-	stream->writeUint32((Uint32)subscribed, "subscribed");
-	stream->writeSint32(caryedRessource, "caryedRessource");
+	stream->writeSint32(caryedRessource, "caryedRessource");	
+	stream->writeSint32(jobTimer, "jobTimer");
+
 	
 	stream->writeLeaveSection();
 }
 
-void Unit::loadCrossRef(GAGCore::InputStream *stream, Team *owner)
+void Unit::loadCrossRef(GAGCore::InputStream *stream, Team *owner, Sint32 versionMinor)
 {
 	stream->readEnterSection("Unit");
 	Uint16 gbid;
@@ -336,12 +341,11 @@ void Unit::loadCrossRef(GAGCore::InputStream *stream, Team *owner)
 		ownExchangeBuilding = NULL;
 	else
 		ownExchangeBuilding = owner->myBuildings[Building::GIDtoID(gbid)];
-	
-	gbid = stream->readUint16("foreingExchangeBuilding");
-	if (gbid == NOGBID)
-		foreingExchangeBuilding = NULL;
-	else
-		foreingExchangeBuilding = owner->myBuildings[Building::GIDtoID(gbid)];
+
+	if(versionMinor < 56)
+	{
+		gbid = stream->readUint16("foreingExchangeBuilding");
+	}
 		
 	stream->readLeaveSection();
 }
@@ -365,32 +369,49 @@ void Unit::saveCrossRef(GAGCore::OutputStream *stream)
 	else
 		stream->writeUint16(NOGBID, "ownExchangeBuilding");
 		
-	if (foreingExchangeBuilding)
-		stream->writeUint16(foreingExchangeBuilding->gid, "foreingExchangeBuilding");
-	else
-		stream->writeUint16(NOGBID, "foreingExchangeBuilding");
-		
 	stream->writeLeaveSection();
 }
 
-void Unit::subscriptionSuccess(void)
+void Unit::subscriptionSuccess(Building* building, bool inside)
 {
+	Building* b=building;
+	
+	if (building->type->isVirtual)
+	{
+		destinationPurprose=-1;
+		fprintf(logFile, "[%d] sdp1 destinationPurprose=%d\n", gid, destinationPurprose);
+		activity=ACT_FLAG;
+		attachedBuilding=b;
+		targetBuilding=b;
+		if (verbose)
+			printf("guid=(%d) unitsWorkingSubscribe(findBestZonable) dp=(%d), gbid=(%d)\n", gid, destinationPurprose, b->gid);
+	}
+	else if(inside == false)
+	{
+		assert(destinationPurprose>=0);
+		assert(b->neededRessource(destinationPurprose));
+		activity=ACT_FILLING;
+		attachedBuilding=b;
+		targetBuilding=NULL;
+		if (verbose)
+			printf("guid=(%d) unitsWorkingSubscribe(findBestZonable) dp=(%d), gbid=(%d)\n", gid, destinationPurprose, b->gid);
+	}
+	else
+	{
+		activity=ACT_UPGRADING;
+		attachedBuilding=b;
+		targetBuilding=b;
+		if (verbose)
+			printf("guid=(%d) unitsWorkingSubscribe(findBestZonable) dp=(%d), gbid=(%d)\n", gid, destinationPurprose, b->gid);
+	}
+
 	if (verbose)
 		printf("guid=(%d), subscriptionSuccess()\n", gid);
-	subscribed=false;
+
 	switch(medical)
 	{
 		case MED_HUNGRY :
 		case MED_DAMAGED :
-		{
-			activity=ACT_UPGRADING;
-			displacement=DIS_GOING_TO_BUILDING;
-			assert(targetBuilding==attachedBuilding);
-			targetX=targetBuilding->getMidX();
-			targetY=targetBuilding->getMidY();
-			validTarget=true;
-		}
-		break;
 		case MED_FREE:
 		{
 			switch(activity)
@@ -424,57 +445,6 @@ void Unit::subscriptionSuccess(void)
 						targetY=targetBuilding->getMidY();
 						validTarget=true;
 					}
-					else if (ownExchangeBuilding)
-					{
-						if (foreingExchangeBuilding)
-						{
-							// Here we are exchanging fruits between ownExchangeBuilding and foreingExchangeBuilding.
-							if (caryedRessource>=HAPPYNESS_BASE
-								&& (foreingExchangeBuilding->receiveRessourceMask & (1<<(caryedRessource-HAPPYNESS_BASE)))
-								&& (foreingExchangeBuilding->ressources[caryedRessource]<foreingExchangeBuilding->type->maxRessource[caryedRessource]))
-							{
-								assert((Uint32)destinationPurprose==(ownExchangeBuilding->receiveRessourceMask & foreingExchangeBuilding->sendRessourceMask));
-								displacement=DIS_GOING_TO_BUILDING;
-								targetBuilding=foreingExchangeBuilding;
-								targetX=targetBuilding->getMidX();
-								targetY=targetBuilding->getMidY();
-								validTarget=true;
-							}
-							else
-							{
-								displacement=DIS_GOING_TO_BUILDING;
-								targetBuilding=ownExchangeBuilding;
-								targetX=targetBuilding->getMidX();
-								targetY=targetBuilding->getMidY();
-								validTarget=true;
-							}
-						}
-						else
-						{
-							assert(false); // Units never get subscription success to fill a food building from an exchange building. (yet)
-							// Here we are taking fruits from ownExchangeBuilding to a food building (attachedBuilding).
-							
-							if (caryedRessource>=HAPPYNESS_BASE
-								&& (attachedBuilding->ressources[caryedRessource]<attachedBuilding->type->maxRessource[caryedRessource]))
-							{
-								assert(destinationPurprose==caryedRessource);
-								displacement=DIS_GOING_TO_BUILDING;
-								targetBuilding=attachedBuilding;
-								targetX=targetBuilding->getMidX();
-								targetY=targetBuilding->getMidY();
-								validTarget=true;
-							}
-							else
-							{
-								assert(destinationPurprose>=HAPPYNESS_BASE);
-								displacement=DIS_GOING_TO_BUILDING;
-								targetBuilding=ownExchangeBuilding;
-								targetX=targetBuilding->getMidX();
-								targetY=targetBuilding->getMidY();
-								validTarget=true;
-							}
-						}
-					}
 					else
 					{
 						displacement=DIS_GOING_TO_RESSOURCE;
@@ -501,6 +471,7 @@ void Unit::subscriptionSuccess(void)
 
 void Unit::syncStep(void)
 {
+	//warrior attacks?
 	assert(speed>0);
 	if ((action==ATTACK_SPEED) && (delta>=128) && (delta<(128+speed)))
 	{
@@ -511,7 +482,7 @@ void Unit::syncStep(void)
 			int enemyTeam=GIDtoTeam(enemyGUID);
 			Unit *enemy=owner->game->teams[enemyTeam]->myUnits[enemyID];
 			
-			int degats=getRealAttackStrength()-enemy->getRealArmor();
+			int degats=getRealAttackStrength()-enemy->getRealArmor(false);
 			if (degats<=0)
 				degats=1;
 			enemy->hp-=degats;
@@ -614,12 +585,10 @@ void Unit::standardRandomActivity()
 	attachedBuilding=NULL;
 	targetBuilding=NULL;
 	ownExchangeBuilding=NULL;
-	foreingExchangeBuilding=NULL;
 	activity=Unit::ACT_RANDOM;
 	displacement=Unit::DIS_RANDOM;
 	validTarget=false;
 	needToRecheckMedical=true;
-	subscribed=false;
 }
 
 void Unit::stopAttachedForBuilding(bool goingInside)
@@ -630,9 +599,7 @@ void Unit::stopAttachedForBuilding(bool goingInside)
 	
 	if (goingInside)
 	{
-		attachedBuilding->unitsInside.remove(this);
-		attachedBuilding->unitsInsideSubscribe.remove(this);
-		
+		attachedBuilding->removeUnitFromInside(this);
 		if (activity==ACT_UPGRADING)
 		{
 			assert(displacement==DIS_GOING_TO_BUILDING);
@@ -644,22 +611,16 @@ void Unit::stopAttachedForBuilding(bool goingInside)
 	{
 		for (std::list<Unit *>::iterator  it=attachedBuilding->unitsInside.begin(); it!=attachedBuilding->unitsInside.end(); ++it)
 			assert(*it!=this);
-		for (std::list<Unit *>::iterator  it=attachedBuilding->unitsInsideSubscribe.begin(); it!=attachedBuilding->unitsInsideSubscribe.end(); ++it)
-			assert(*it!=this);
 	}
 	
 	activity=ACT_RANDOM;
 	displacement=DIS_RANDOM;
 	validTarget=false;
 	
-	attachedBuilding->unitsWorking.remove(this);
-	attachedBuilding->unitsWorkingSubscribe.remove(this);
-	attachedBuilding->updateCallLists();
+	attachedBuilding->removeUnitFromWorking(this);
 	attachedBuilding=NULL;
 	targetBuilding=NULL;
 	ownExchangeBuilding=NULL;
-	foreingExchangeBuilding=NULL;
-	subscribed=false;
 	assert(needToRecheckMedical);
 }
 
@@ -708,7 +669,7 @@ void Unit::handleMagic(void)
 						if (owner->enemies & targetTeamMask)
 						{
 							Unit *enemyUnit = teams[targetTeam]->myUnits[targetID];
-							Sint32 damage = attackForce + experienceLevel - enemyUnit->getRealArmor();
+							Sint32 damage = attackForce + experienceLevel - enemyUnit->getRealArmor(true);
 							if (damage > 0)
 							{
 								enemyUnit->hp -= damage;
@@ -721,8 +682,9 @@ void Unit::handleMagic(void)
 					}
 				}
 				
-				// damaging enemy buildings:
-				if (performance[MAGIC_ATTACK_GROUND])
+				// damaging enemy buildings: this has been removed for balance purposes
+				
+				/*if (performance[MAGIC_ATTACK_GROUND])
 				{
 					Uint16 targetGBID = map->getBuilding(xi, yi);
 					if (damagedBuildings.insert(targetGBID).second)
@@ -746,7 +708,7 @@ void Unit::handleMagic(void)
 							}
 						}
 					}
-				}
+				}*/
 			}
 		
 		Sint32 magicLevel = std::max(level[MAGIC_ATTACK_AIR], level[MAGIC_ATTACK_GROUND]);
@@ -784,16 +746,11 @@ void Unit::handleMedical(void)
 			if (attachedBuilding)
 			{
 				assert((displacement!=DIS_ENTERING_BUILDING) && (displacement!=DIS_INSIDE) && (displacement!=DIS_EXITING_BUILDING));
-				attachedBuilding->unitsWorking.remove(this);
-				attachedBuilding->unitsInside.remove(this);
-				attachedBuilding->unitsWorkingSubscribe.remove(this);
-				attachedBuilding->unitsInsideSubscribe.remove(this);
-				attachedBuilding->updateCallLists();
+				attachedBuilding->removeUnitFromWorking(this);
+				attachedBuilding->removeUnitFromInside(this);
 				attachedBuilding=NULL;
 				targetBuilding=NULL;
 				ownExchangeBuilding=NULL;
-				foreingExchangeBuilding=NULL;
-				subscribed=false;
 			}
 			
 			activity=ACT_RANDOM;
@@ -822,179 +779,61 @@ void Unit::handleActivity(void)
 	if (verbose)
 		printf("guid=(%d) handleActivity (medical=%d, activity=%d) (needToRecheckMedical=%d) (attachedBuilding=%p)...\n",
 			gid, medical, activity, needToRecheckMedical, attachedBuilding);
-	
+			
+	if(activity!=ACT_RANDOM)
+		jobTimer=0;
+
 	if (medical==MED_FREE)
 	{
 		handleMagic();
-		
+
 		if (activity==ACT_RANDOM)
 		{
-			// look for a "job"
-			// else keep walking around
-			if (verbose)
-				printf("guid=(%d) looking for a job...\n", gid);
-
-			// first we look for a food building to fill, because it is the first priority.
-			// FIXME: Ugly hacks added by Andrew to make sure that globs only subscribe to an inn they have a chance of being accepted at
-			if (performance[HARVEST])
-			{
-				Building *b=owner->findBestFoodable(this);
-
-				int dist;
-				int timeLeft = numberOfStepsLeftUntilHungry();
-				// Is there a building we can reach in time?
-				if (b!=NULL && owner->map->buildingAvailable(b, performance[SWIM], posX, posY, &dist) && dist < timeLeft)
-				{
-					bool canSubscribe=(caryedRessource>=0) && b->neededRessource(caryedRessource);
-
-					if (!canSubscribe)
-					{
-						int needs[MAX_NB_RESSOURCES];
-						b->wishedRessources(needs);
-						for (int r=0; r<MAX_RESSOURCES; r++)
-							if (needs[r]>0 && owner->map->ressourceAvailable(owner->teamNumber, r, performance[SWIM], posX, posY, &dist) && (dist<timeLeft))
-							{
-								canSubscribe=true;
-								break;
-							};
-					};
-
-					if (canSubscribe)
-					{
-						assert(destinationPurprose>=0);
-						assert(b->neededRessource(destinationPurprose));
-						activity=ACT_FILLING;
-						attachedBuilding=b;
-						targetBuilding=NULL;
-						if (verbose)
-							printf("guid=(%d) unitsWorkingSubscribe(findBestZonable) dp=(%d), gbid=(%d)\n", gid, destinationPurprose, b->gid);
-						b->unitsWorkingSubscribe.push_front(this);
-						subscribed=true;
-						if (b->subscribeToBringRessources!=1)
-							{
-								b->subscribeToBringRessources=1;
-								owner->subscribeToBringRessources.push_front(b);
-							}
-						if (b->subscriptionWorkingTimer<=0)
-							b->subscriptionWorkingTimer=1;
-						return;
-					}
-				}
-			}
-			
-			// second we go to flag
-			Building *b;
-			b=owner->findBestZonable(this);
-			if (b)
-			{
-				destinationPurprose=-1;
-				fprintf(logFile, "[%d] sdp1 destinationPurprose=%d\n", gid, destinationPurprose);
-				activity=ACT_FLAG;
-				attachedBuilding=b;
-				targetBuilding=b;
-				if (verbose)
-					printf("guid=(%d) unitsWorkingSubscribe(findBestZonable) dp=(%d), gbid=(%d)\n", gid, destinationPurprose, b->gid);
-				b->unitsWorkingSubscribe.push_front(this);
-				subscribed=true;
-				if (b->subscribeForFlaging!=1)
-				{
-					b->subscribeForFlaging=1;
-					owner->subscribeForFlaging.push_back(b);
-				}
-				if (b->subscriptionWorkingTimer<=0)
-					b->subscriptionWorkingTimer=1;
-				return;
-			}
-
-			// third we look for upgrade
-			b=owner->findBestUpgrade(this);
-			if (b)
-			{
-				assert(destinationPurprose>=WALK);
-				assert(destinationPurprose<ARMOR);
-				activity=ACT_UPGRADING;
-				attachedBuilding=b;
-				targetBuilding=b;
-				if (verbose)
-					printf("guid=(%d) going to upgrade at dp=(%d), gbid=(%d)\n", gid, destinationPurprose, b->gid);
-				b->unitsInsideSubscribe.push_front(this);
-				subscribed=true;
-				if (b->subscribeForInside!=1)
-				{
-					b->subscribeForInside=1;
-					owner->subscribeForInside.push_front(b);
-				}
-				if (b->subscriptionInsideTimer<=0)
-					b->subscriptionInsideTimer=1;
-				return;
-			}
-			
-			// fourth we harvest for construction, or other lower priority.
-			if (performance[HARVEST])
-			{
-				// if we have a ressource
-				Building *b=owner->findBestFillable(this);
-				if (b)
-				{
-					assert(b->type->canExchange || (destinationPurprose>=0 && b->neededRessource(destinationPurprose)));
-					activity=ACT_FILLING;
-					attachedBuilding=b;
-					targetBuilding=NULL;
-					if (verbose)
-						printf("guid=(%d) unitsWorkingSubscribe(findBestFillable) dp=(%d), gbid=(%d)\n", gid, destinationPurprose, b->gid);
-					b->unitsWorkingSubscribe.push_front(this);
-					subscribed=true;
-					if (b->subscribeToBringRessources!=1)
-					{
-						b->subscribeToBringRessources=1;
-						owner->subscribeToBringRessources.push_front(b);
-					}
-					if (b->subscriptionWorkingTimer<=0)
-						b->subscriptionWorkingTimer=1;
-					return;
-				}
-			}
-			
-			if (verbose)
-				printf("guid=(%d) no job found.\n", gid);
-			
 			// nothing to do:
-			// we go to a heal building if we'r not fully healed: (1/8 trigger)
-			if (hp+(performance[HP]/10) < performance[HP])
+			//Wait for 32 ticks before doing something else, to allow buildings time to hire units
+			jobTimer++;
+			if(jobTimer>32)
 			{
-				Building *b;
-				b=owner->findNearestHeal(this);
+				// We look for an upgrade
+				Building* b=owner->findBestUpgrade(this);
 				if (b)
 				{
-					destinationPurprose=HEAL;
-					fprintf(logFile, "[%d] sdp2 destinationPurprose=%d\n", gid, destinationPurprose);
+					assert(destinationPurprose>=WALK);
+					assert(destinationPurprose<ARMOR);
 					activity=ACT_UPGRADING;
 					attachedBuilding=b;
 					targetBuilding=b;
-					needToRecheckMedical=false;
 					if (verbose)
-						printf("guid=(%d) Going to heal building\n", gid);
-					targetX=attachedBuilding->getMidX();
-					targetY=attachedBuilding->getMidY();
-					validTarget=true;
-					b->unitsInsideSubscribe.push_front(this);
-					subscribed=true;
-					if (b->subscribeForInside!=1)
+						printf("guid=(%d) going to upgrade at dp=(%d), gbid=(%d)\n", gid, destinationPurprose, b->gid);
+					b->subscribeUnitForInside(this);
+					return;
+				}
+
+				// we go to a heal building if we'r not fully healed: (1/8 trigger)
+				if (hp+(performance[HP]/10) < performance[HP])
+				{
+					Building *b;
+					b=owner->findNearestHeal(this);
+					if (b)
 					{
-						b->subscribeForInside=1;
-						owner->subscribeForInside.push_front(b);
+						destinationPurprose=HEAL;
+						fprintf(logFile, "[%d] sdp2 destinationPurprose=%d\n", gid, destinationPurprose);
+						activity=ACT_UPGRADING;
+						attachedBuilding=b;
+						targetBuilding=b;
+						needToRecheckMedical=false;
+						if (verbose)
+							printf("guid=(%d) Going to heal building\n", gid);
+						targetX=attachedBuilding->getMidX();
+						targetY=attachedBuilding->getMidY();
+						validTarget=true;
+						b->subscribeUnitForInside(this);
 					}
-					if (b->subscriptionInsideTimer<=0)
-						b->subscriptionInsideTimer=1;
+					else
+						activity=ACT_RANDOM;
 				}
-				else
-					activity=ACT_RANDOM;
 			}
 		}
-		else
-		{
-			// we keep the job
-		}
 	}
 	else if (needToRecheckMedical)
 	{
@@ -1002,16 +841,11 @@ void Unit::handleActivity(void)
 		{
 			if (verbose)
 				printf("guid=(%d) Need medical while working, abort work\n", gid);
-			attachedBuilding->unitsWorking.remove(this);
-			attachedBuilding->unitsInside.remove(this);
-			attachedBuilding->unitsWorkingSubscribe.remove(this);
-			attachedBuilding->unitsInsideSubscribe.remove(this);
-			attachedBuilding->updateCallLists();
+			attachedBuilding->removeUnitFromWorking(this);
+			attachedBuilding->removeUnitFromInside(this);
 			attachedBuilding=NULL;
 			targetBuilding=NULL;
 			ownExchangeBuilding=NULL;
-			foreingExchangeBuilding=NULL;
-			subscribed=false;
 		}
 
 		if (medical==MED_HUNGRY)
@@ -1074,15 +908,7 @@ void Unit::handleActivity(void)
 				needToRecheckMedical=false;
 				if (verbose)
 					printf("guid=(%d) Subscribed to food at building gbid=(%d)\n", gid, b->gid);
-				b->unitsInsideSubscribe.push_front(this);
-				subscribed=true;
-				if (b->subscribeForInside!=1)
-				{
-					b->subscribeForInside=1;
-					owner->subscribeForInside.push_front(b);
-				}
-				if (b->subscriptionInsideTimer<=0)
-					b->subscriptionInsideTimer=1;
+				b->subscribeUnitForInside(this);
 			}
 			else
 				activity=ACT_RANDOM;
@@ -1101,15 +927,7 @@ void Unit::handleActivity(void)
 				needToRecheckMedical=false;
 				if (verbose)
 					printf("guid=(%d) Subscribed to heal at building gbid=(%d)\n", gid, b->gid);
-				b->unitsInsideSubscribe.push_front(this);
-				subscribed=true;
-				if (b->subscribeForInside!=1)
-				{
-					b->subscribeForInside=1;
-					owner->subscribeForInside.push_front(b);
-				}
-				if (b->subscriptionInsideTimer<=0)
-					b->subscriptionInsideTimer=1;
+				b->subscribeUnitForInside(this);
 			}
 			else
 				activity=ACT_RANDOM;
@@ -1121,14 +939,7 @@ void Unit::handleActivity(void)
 
 void Unit::handleDisplacement(void)
 {
-	if (subscribed)
-	{
-		if (verbose)
-			printf("guid=(%d) handleDisplacement() subscribed, then displacement=DIS_RANDOM\n", gid);
-		displacement=DIS_RANDOM;
-		validTarget=false;
-	}
-	else switch (activity)
+	switch (activity)
 	{
 		case ACT_RANDOM:
 		{
@@ -1198,121 +1009,10 @@ void Unit::handleDisplacement(void)
 				bool loopMove=false;
 				bool exchangeReady=false;
 				assert(targetBuilding);
-				
-				if (foreingExchangeBuilding && (targetBuilding==foreingExchangeBuilding))
-				{
-					// Here we are aside a foreing exchange building, to drop our own ressource and take a foreign ressource:
-					assert(targetBuilding);
-					assert(ownExchangeBuilding);
-					assert(foreingExchangeBuilding);
-					assert(targetBuilding->type->canExchange);
-					assert(ownExchangeBuilding->type->canExchange);
-					assert(foreingExchangeBuilding->type->canExchange);
-					assert(owner!=targetBuilding->owner);
-					assert(owner==ownExchangeBuilding->owner);
-					assert(owner!=foreingExchangeBuilding->owner);
-					
-					assert(caryedRessource>=HAPPYNESS_BASE);
-					// Let's check for possible exchange ressources:
-					
-					bool goodToGive=targetBuilding->ressources[caryedRessource]<targetBuilding->type->maxRessource[caryedRessource];
-					int ressourceToTake=-1;
-					for (int r=0; r<HAPPYNESS_COUNT; r++)
-						if ((destinationPurprose & (1<<r))
-							&& foreingExchangeBuilding->ressources[HAPPYNESS_BASE+r]>0)
-						{
-							ressourceToTake=HAPPYNESS_BASE+r;
-							break;
-						}
-					
-					if (goodToGive && (ressourceToTake>=0))
-					{
-						// OK, we can proceed to the exchange.
-						targetBuilding->ressources[caryedRessource]+=targetBuilding->type->multiplierRessource[caryedRessource];
-						targetBuilding->ressources[ressourceToTake]-=targetBuilding->type->multiplierRessource[ressourceToTake];
-						
-						if (targetBuilding->ressources[caryedRessource]>targetBuilding->type->maxRessource[caryedRessource])
-							targetBuilding->ressources[caryedRessource]=targetBuilding->type->maxRessource[caryedRessource];
-					
-						if (targetBuilding->ressources[ressourceToTake]<0)
-							targetBuilding->ressources[ressourceToTake]=0;
-					
-						caryedRessource=ressourceToTake;
-						targetBuilding=ownExchangeBuilding;
-						
-						displacement=DIS_GOING_TO_BUILDING;
-						targetX=targetBuilding->getMidX();
-						targetY=targetBuilding->getMidY();
-						validTarget=true;
-						exchangeReady=true;
-						if (verbose)
-							printf("guid=(%d) exchangeReady at foreingExchangeBuilding\n", gid);
-					}
-					else
-					{
-						if (verbose)
-							printf("guid=(%d) loopMove at foreingExchangeBuilding, caryedRessource=%d, destinationPurprose=%d, goodToGive=%d, goodToTake=%d\n", gid, caryedRessource, destinationPurprose, goodToGive, (ressourceToTake>=0));
-						loopMove=true;
-					}
-				}
-				else if (foreingExchangeBuilding && (targetBuilding==ownExchangeBuilding))
-				{
-					// Here we are aside our own exchange building, to take one of our ressource and maybe to drop a (foreign) ressource.
-					assert(targetBuilding);
-					assert(ownExchangeBuilding);
-					assert(foreingExchangeBuilding);
-					assert(targetBuilding->type->canExchange);
-					assert(ownExchangeBuilding->type->canExchange);
-					assert(foreingExchangeBuilding->type->canExchange);
-					assert(owner==targetBuilding->owner);
-					assert(owner==ownExchangeBuilding->owner);
-					assert(owner!=foreingExchangeBuilding->owner);
-					
-					if (caryedRessource>=0)
-					{
-						targetBuilding->ressources[caryedRessource]+=targetBuilding->type->multiplierRessource[caryedRessource];
-						if (targetBuilding->ressources[caryedRessource]>targetBuilding->type->maxRessource[caryedRessource])
-							targetBuilding->ressources[caryedRessource]=targetBuilding->type->maxRessource[caryedRessource];
-						caryedRessource=-1;
-					}
-					
-					// Let's grab the right ressource.
-					int ressourceToTake=-1;
-					for (int r=0; r<HAPPYNESS_COUNT; r++)
-						if (foreingExchangeBuilding->receiveRessourceMask & (1<<r))
-						{
-							ressourceToTake=HAPPYNESS_BASE+r;
-							break;
-						}
-					
-					if (ressourceToTake>=0)
-					{
-						int foreignBuildingDist;
-						int timeLeft = numberOfStepsLeftUntilHungry();
-						if (owner->map->buildingAvailable(foreingExchangeBuilding, performance[SWIM], posX, posY, &foreignBuildingDist)
-							&& (foreignBuildingDist<(timeLeft>>1)))
-						{
-							targetBuilding->ressources[ressourceToTake]-=targetBuilding->type->multiplierRessource[ressourceToTake];
-							if (targetBuilding->ressources[ressourceToTake]<0)
-								targetBuilding->ressources[ressourceToTake]=0;
-							caryedRessource=ressourceToTake;
-
-							targetBuilding=foreingExchangeBuilding;
-							displacement=DIS_GOING_TO_BUILDING;
-							targetX=targetBuilding->getMidX();
-							targetY=targetBuilding->getMidY();
-							validTarget=true;
-							exchangeReady=true;
-							if (verbose)
-								printf("guid=(%d) exchangeReady at ownExchangeBuilding\n", gid);
-						}
-					}
-				}
-				else if ((foreingExchangeBuilding==NULL) && (targetBuilding==ownExchangeBuilding))
+				if (targetBuilding==ownExchangeBuilding)
 				{
 					assert(targetBuilding);
 					assert(ownExchangeBuilding);
-					assert(foreingExchangeBuilding==NULL);
 					assert(targetBuilding->type->canExchange);
 					assert(ownExchangeBuilding->type->canExchange);
 					assert(owner==targetBuilding->owner);
@@ -1326,9 +1026,7 @@ void Unit::handleDisplacement(void)
 					
 					if (targetBuilding->ressources[destinationPurprose]>0)
 					{
-						targetBuilding->ressources[destinationPurprose]-=targetBuilding->type->multiplierRessource[destinationPurprose];
-						if (targetBuilding->ressources[destinationPurprose]<0)
-							targetBuilding->ressources[destinationPurprose]=0;
+						targetBuilding->removeRessourceFromBuilding(destinationPurprose);
 						caryedRessource=destinationPurprose;
 						fprintf(logFile, "[%d] sdp6 destinationPurprose=%d\n", gid, destinationPurprose);
 						
@@ -1346,42 +1044,13 @@ void Unit::handleDisplacement(void)
 				{
 					if (verbose)
 						printf("guid=(%d) Giving ressource (%d) to building gbid=(%d) old-amount=(%d)\n", gid, destinationPurprose, targetBuilding->gid, targetBuilding->ressources[caryedRessource]);
-					targetBuilding->ressources[caryedRessource]+=targetBuilding->type->multiplierRessource[caryedRessource];
-					if (targetBuilding->ressources[caryedRessource]>targetBuilding->type->maxRessource[caryedRessource])
-						targetBuilding->ressources[caryedRessource]=targetBuilding->type->maxRessource[caryedRessource];
+					targetBuilding->addRessourceIntoBuilding(caryedRessource);
 					caryedRessource=-1;
-					BuildingType *bt=targetBuilding->type;
-					switch (targetBuilding->constructionResultState)
-					{
-						case Building::NO_CONSTRUCTION:
-						break;
 
-						case Building::NEW_BUILDING:
-						case Building::UPGRADE:
-						{
-							attachedBuilding->hp+=bt->hpInc;
-							if (attachedBuilding->hp > bt->hpMax)
-								attachedBuilding->hp = bt->hpMax;
-						}
-						break;
-
-						case Building::REPAIR:
-						{
-							int totRessources=0;
-							for (unsigned i=0; i<MAX_NB_RESSOURCES; i++)
-								totRessources+=bt->maxRessource[i];
-							attachedBuilding->hp+=bt->hpMax/totRessources;
-						}
-						break;
-
-						default:
-							assert(false);
-					}
 				}
 				
 				if (!loopMove && !exchangeReady)
 				{
-					attachedBuilding->update();
 					//NOTE: if attachedBuilding has become NULL; it's beacause the building doesn't need me anymore.
 					if (!attachedBuilding)
 					{
@@ -1390,11 +1059,14 @@ void Unit::handleDisplacement(void)
 						activity=ACT_RANDOM;
 						displacement=DIS_RANDOM;
 						validTarget=false;
-						subscribed=false;
 						assert(needToRecheckMedical);
 					}
 					else
 					{
+						///Find a ressource that the building wants and a location to get it from
+						///The location may be a market, or the harvesting the ressource from the
+						///map.
+						attachedBuilding->update();
 						int needs[MAX_NB_RESSOURCES];
 						attachedBuilding->wishedRessources(needs);
 						int teamNumber=owner->teamNumber;
@@ -1439,7 +1111,6 @@ void Unit::handleDisplacement(void)
 														minValue=value;
 
 														ownExchangeBuilding=*bi;
-														foreingExchangeBuilding=NULL;
 														targetBuilding=*bi;
 														takeInExchangeBuilding=true;
 													}
@@ -1484,66 +1155,6 @@ void Unit::handleDisplacement(void)
 									}
 								}
 							}
-							else if (targetBuilding!=foreingExchangeBuilding && attachedBuilding->type->canExchange && owner->openMarket())
-							{
-								// Here we try to start a new exchange with a foreign exchange building:
-								assert(owner==targetBuilding->owner);
-								if (ownExchangeBuilding)
-								{
-									assert(owner==ownExchangeBuilding->owner);
-									assert(ownExchangeBuilding==attachedBuilding);
-								}
-								if (foreingExchangeBuilding)
-									assert(owner!=foreingExchangeBuilding->owner);
-
-								ownExchangeBuilding=attachedBuilding;
-
-								foreingExchangeBuilding=NULL;
-								int minDist=INT_MAX;
-
-								Uint32 sendRessourceMask=ownExchangeBuilding->sendRessourceMask;
-								Uint32 receiveRessourceMask=ownExchangeBuilding->receiveRessourceMask;
-								for (int ti=0; ti<owner->game->session.numberOfTeam; ti++)
-									if (ti!=teamNumber && (owner->game->teams[ti]->sharedVisionExchange & owner->me))
-									{
-										Team *foreignTeam=owner->game->teams[ti];
-										std::list<Building *> foreignFillable=foreignTeam->fillable;
-										for (std::list<Building *>::iterator fbi=foreignFillable.begin(); fbi!=foreignFillable.end(); ++fbi)
-											if ((*fbi)->type->canExchange)
-											{
-												Uint32 foreignSendRessourceMask=(*fbi)->sendRessourceMask;
-												Uint32 foreignReceiveRessourceMask=(*fbi)->receiveRessourceMask;
-												int foreignBuildingDist;
-												if ((sendRessourceMask & foreignReceiveRessourceMask)
-													&& (receiveRessourceMask & foreignSendRessourceMask)
-													&& map->buildingAvailable(*fbi, canSwim, posX, posY, &foreignBuildingDist)
-													&& foreignBuildingDist<(timeLeft>>1)
-													&& foreignBuildingDist<minDist)
-												{
-													foreingExchangeBuilding=*fbi;
-													minDist=foreignBuildingDist;
-													destinationPurprose=receiveRessourceMask & foreignSendRessourceMask;
-													fprintf(logFile, "[%d] sdp8 destinationPurprose=%d\n", gid, destinationPurprose);
-												}
-											}
-									}
-
-								if (foreingExchangeBuilding)
-								{
-									assert(activity==ACT_FILLING);
-									displacement=DIS_GOING_TO_BUILDING;
-									targetBuilding=ownExchangeBuilding;
-									targetX=targetBuilding->getMidX();
-									targetY=targetBuilding->getMidY();
-									validTarget=true;
-								}
-								else
-								{
-									if (verbose)
-										printf("guid=(%d) can't find any suitable foreign exchange building, unsubscribing.\n", gid);
-									stopAttachedForBuilding(false);
-								}
-							}
 							else 
 							{
 								if (verbose)
@@ -1947,7 +1558,7 @@ void Unit::handleMovement(void)
 							{
 								int id=Building::GIDtoID(gid);
 								Unit *u=owner->game->teams[team]->myUnits[id];
-								if (((owner->sharedVisionExchange & tm)==0) || (u->foreingExchangeBuilding==NULL))
+								if (((owner->sharedVisionExchange & tm)==0))
 								{
 									int attackStrength=u->getRealAttackStrength();
 									int newQuality=((x*x+y*y)<<8)/(1+attackStrength);
@@ -2176,15 +1787,11 @@ void Unit::handleMovement(void)
 				activity=ACT_RANDOM;
 				movement=MOV_EXITING_BUILDING;
 				fprintf(logFile, "guid=(%d) exiting gbid=%d\n", gid, attachedBuilding->gid);
-				attachedBuilding->unitsInside.remove(this);
-				attachedBuilding->unitsInsideSubscribe.remove(this);
-				attachedBuilding->updateCallLists();
+				attachedBuilding->removeUnitFromInside(this);
 				attachedBuilding->updateConstructionState();
 				attachedBuilding=NULL;
 				targetBuilding=NULL;
 				assert(ownExchangeBuilding==NULL);
-				assert(foreingExchangeBuilding==NULL);
-				subscribed=false;
 				assert(needToRecheckMedical);
 			}
 			else
@@ -2285,6 +1892,10 @@ void Unit::handleAction(void)
 			assert(!performance[FLY]);
 			owner->map->setGroundUnit(posX, posY, NOGUID);
 			gotoGroundTarget();
+			if(dx==0 && dy==0)
+			owner->map->addHiddenForbidden(posX, posY, owner->teamNumber);
+			else
+				owner->map->removeHiddenForbidden(posX, posY, owner->teamNumber);
 			posX=(posX+dx)&(owner->map->getMaskW());
 			posY=(posY+dy)&(owner->map->getMaskH());
 			selectPreferedGroundMovement();
@@ -2319,6 +1930,12 @@ void Unit::handleAction(void)
 				owner->map->setGroundUnit(posX, posY, NOGUID);
 			
 			directionFromDxDy();
+			
+			if(dx==0 && dy==0)
+				owner->map->addHiddenForbidden(posX, posY, owner->teamNumber);
+			else
+				owner->map->removeHiddenForbidden(posX, posY, owner->teamNumber);
+				
 			posX=(posX+dx)&(owner->map->getMaskW());
 			posY=(posY+dy)&(owner->map->getMaskH());
 			
@@ -2399,6 +2016,8 @@ void Unit::handleAction(void)
 			directionFromDxDy();
 			action=HARVEST;
 			speed=performance[action];
+			if(speed==0)
+				speed/=speed;
 			break;
 		}
 		
@@ -2709,10 +2328,18 @@ Uint16 Unit::GIDfrom(Sint32 id, Sint32 team)
 }
 
 //! Return the real armor, taking into account the reduction due to fruits
-int Unit::getRealArmor(void) const
+int Unit::getRealArmor(bool isMagic) const
 {
-	int armorReductionPerHappyness = race->getUnitType(typeNum, level[ARMOR])->armorReductionPerHappyness;
-	return performance[ARMOR] - fruitCount * armorReductionPerHappyness;
+	if (isMagic == false)
+	{
+		int armorReductionPerHappyness = race->getUnitType(typeNum, level[ARMOR])->armorReductionPerHappyness;
+		return performance[ARMOR] - fruitCount * armorReductionPerHappyness;
+	}
+	else if(isMagic == true) //magic bypasses armor yet fruit penalties still apply
+	{
+		int armorReductionPerHappyness = race->getUnitType(typeNum, level[ARMOR])->armorReductionPerHappyness;
+		return 0 - fruitCount * armorReductionPerHappyness;
+	}
 }
 
 //! Return the real attack strengh, taking into account the experience level
@@ -2809,8 +2436,6 @@ void Unit::integrity()
 	if (!needToRecheckMedical)
 	{
 		assert(activity==ACT_UPGRADING);
-		if (!subscribed) 
-			assert(displacement==DIS_GOING_TO_BUILDING || displacement==DIS_ENTERING_BUILDING || displacement==DIS_INSIDE);
 		assert(destinationPurprose==HEAL || destinationPurprose==FEED);
 	}
 }
@@ -2942,17 +2567,11 @@ Uint32 Unit::checkSum(std::vector<Uint32> *checkSumsVector)
 	cs^=(ownExchangeBuilding!=NULL ? 2:0);
 	if (checkSumsVector)
 		checkSumsVector->push_back((ownExchangeBuilding!=NULL ? 1:0));// [29]
-	cs^=(foreingExchangeBuilding!=NULL ? 4:0);
-	if (checkSumsVector)
-		checkSumsVector->push_back((foreingExchangeBuilding!=NULL ? 1:0));// [30]
 	cs=(cs<<1)|(cs>>31);
 	
 	cs^=destinationPurprose;
 	if (checkSumsVector)
 		checkSumsVector->push_back(destinationPurprose);// [31]
-	cs^=(Uint32)subscribed;
-	if (checkSumsVector)
-		checkSumsVector->push_back(subscribed);// [32]
 	cs^=caryedRessource;
 	if (checkSumsVector)
 		checkSumsVector->push_back(caryedRessource);// [33]
