@@ -53,10 +53,13 @@ public:
 	
 	void load(GAGCore::InputStream *stream, Team *owner, Sint32 versionMinor);
 	void save(GAGCore::OutputStream *stream);
-	void loadCrossRef(GAGCore::InputStream *stream, Team *owner);
+	void loadCrossRef(GAGCore::InputStream *stream, Team *owner, Sint32 versionMinor);
 	void saveCrossRef(GAGCore::OutputStream *stream);
 	
-	void subscriptionSuccess(void); //Called by the building the unit has subscribed to.
+	///This function is called by a Building that has subscribed this unit.
+	///If the unit has been subscribed for upgrading or for food, as opposed
+	///to being subscribed for work, inside is set to true.
+	void subscriptionSuccess(Building* building, bool inside);
 	void syncStep(void);
 	
 	void directionFromDxDy(void);
@@ -88,7 +91,7 @@ public:
 	bool isUnitHungry(void);
 	void standardRandomActivity();
 	
-	int getRealArmor(void) const;
+	int getRealArmor(bool isMagic) const;
 	int getRealAttackStrength(void) const; //!< Return the real attack strengh for warriors
 	int getNextLevelThreshold(void) const;
 	void incrementExperience(int increment);
@@ -237,10 +240,11 @@ public:
 	Building *attachedBuilding;
 	Building *targetBuilding;
 	Building *ownExchangeBuilding;
-	Building *foreingExchangeBuilding;
 	Sint32 destinationPurprose;
-	bool subscribed;
 	int caryedRessource;
+	/// This counts 32 ticks to wait for a job before a unit goes off
+	/// to upgrade or heal when it is otherwise doing nothing.
+	Sint32 jobTimer;
 	
 	// gui
 	int levelUpAnimation;
